@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: call.cpp,v 1.62 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: call.cpp,v 1.92 2008/10/05 18:39:46 bowkenken Exp $
 ***************************************************************/
 
 #define	CALL_CPP
@@ -35,6 +35,51 @@
 extern WSCwindow *StatWin;
 #endif
 
+static long g_joy_n = 0;
+static joy_kind_t g_joy_kind = JOY_KIND_PS;
+
+static long g_win_x[WIN_KIND_MAX_N];
+static long g_win_y[WIN_KIND_MAX_N];
+static long g_win_w[WIN_KIND_MAX_N];
+static long g_win_h[WIN_KIND_MAX_N];
+
+/***************************************************************
+* GUI åɤΥå
+***************************************************************/
+
+void	gui_begin( void )
+{
+#ifdef D_WS
+	WSGIappDev()->lock();
+#endif // D_WS
+
+#ifdef D_GTK
+	gdk_threads_enter();
+#endif // D_GTK
+
+#ifdef D_MFC
+#endif // D_MFC
+}
+
+/***************************************************************
+* GUI åɤΥå
+***************************************************************/
+
+void	gui_end( void )
+{
+#ifdef D_WS
+	WSGIappDev()->unlock();
+#endif // D_WS
+
+#ifdef D_GTK
+	gdk_flush();
+	gdk_threads_leave();
+#endif // D_GTK
+
+#ifdef D_MFC
+#endif // D_MFC
+}
+
 /***************************************************************
 * ᥤ󡦥롼פΥåɤڤ
 * void *(*func)( void * ) : 롦Хåؿ
@@ -74,6 +119,17 @@ bool_t	call_game_thread_exit(
 	return TRUE;
 }
 
+/***************************************************************
+* ޡͭˤ
+***************************************************************/
+
+void	call_ena_timer( void )
+{
+#ifdef D_GTK
+	ena_timer();
+#endif // D_GTK
+}
+
 /***************************************************************
 * ޥåסɥϢ³¹ԥܥΥå
 * bool_t flagExec : 
@@ -81,27 +137,23 @@ bool_t	call_game_thread_exit(
 
 void	call_gui_map_cont_exec_set( bool_t flagExec )
 {
-#ifdef D_WS
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-#endif // D_WS
-
 #ifdef D_GTK
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	gdk_threads_enter();
+	gui_begin();
 
-	gtk_toggle_button_set_active(
-			GTK_TOGGLE_BUTTON( gMapButtonContExec ),
-			flagExec );
+	if( flagExec ){
+		set_label_text_button( GTK_BUTTON( gMapButtonContExec ),
+				MSG_GUI_MAP_CONT_EXEC_ON, 0.5, 0.5 );
+	} else {
+		set_label_text_button( GTK_BUTTON( gMapButtonContExec ),
+				MSG_GUI_MAP_CONT_EXEC_OFF, 0.5, 0.5 );
+	}
 
-	gdk_flush();
-	gdk_threads_leave();
+	gui_end();
 #endif // D_GTK
 }
 
@@ -112,41 +164,18 @@ void	call_gui_map_cont_exec_set( bool_t flagExec )
 
 void	call_gui_menu_draw( draw_menu_t *data )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiMenu.draw( data );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gGuiMenu.draw( data );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiMenu.draw( data );
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -155,41 +184,18 @@ void	call_gui_menu_draw( draw_menu_t *data )
 
 void	call_gui_menu_wipe( void )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiMenu.wipe();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gGuiMenu.wipe();
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiMenu.wipe();
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -199,32 +205,20 @@ void	call_gui_menu_wipe( void )
 
 void	call_set_mouse_dflt_menu( long sel_n )
 {
-#ifdef D_WS
+#ifndef D_MFC
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiMenu.setMouseDfltMenu( sel_n );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gGuiMenu.setMouseDfltMenu( sel_n );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+	gui_end();
+#endif
+#endif // ! D_MFC
 }
 
 /***************************************************************
@@ -233,31 +227,18 @@ void	call_set_mouse_dflt_menu( long sel_n )
 
 void	call_gui_message_set_flag_more( bool_t flag )
 {
-#ifdef D_WS
-#endif // D_WS
-
-#ifdef D_GTK
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	gdk_threads_enter();
-
-	gGuiMessage.setFlagMore( flag );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
+	gui_begin();
 
 	gGuiMessage.setFlagMore( (bool)(flag != FALSE) );
-#endif // D_MFC
+
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -266,43 +247,19 @@ void	call_gui_message_set_flag_more( bool_t flag )
 
 void	call_gui_message_draw( void )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
-
-//@@@	gGuiMessage.draw();
-
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
+	gui_begin();
 
 	gGuiMessage.draw( false );
 	gGuiMessage.drawScrollBar();
 
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiMessage.draw( false );
-	gGuiMessage.drawScrollBar();
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -311,41 +268,18 @@ void	call_gui_message_draw( void )
 
 void	call_gui_num_draw( long *n, long min, long max, long figure )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiNum.draw( n, min, max, figure );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gGuiNum.draw( n, min, max, figure );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiNum.draw( n, min, max, figure );
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -354,41 +288,18 @@ void	call_gui_num_draw( long *n, long min, long max, long figure )
 
 void	call_gui_num_wipe( void )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiNum.wipe();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gGuiNum.wipe();
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiNum.wipe();
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -399,44 +310,18 @@ void	call_gui_input_string_draw(
 	const char *ttl, const char *str, long len
 )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiInpStr.draw( ttl, str, len );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-#if 0
-//@@@
-	gGuiInpStr.draw( ttl, str, len );
+	gui_end();
 #endif
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiInpStr.draw( ttl, str, len );
-#endif // D_MFC
 }
 
 /***************************************************************
@@ -445,44 +330,261 @@ void	call_gui_input_string_draw(
 
 void	call_gui_input_string_wipe( void )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gGuiInpStr.wipe();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-#if 0
-//@@@
-	gGuiInpStr.wipe();
+	gui_end();
 #endif
+}
 
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+/***************************************************************
+* 쥦ɥ
+* const char *str : 
+***************************************************************/
 
-#ifdef D_MFC
+void	call_gui_words_draw( const char *str )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	gGuiInpStr.wipe();
-#endif // D_MFC
+	gui_begin();
+
+	gGuiWords.draw( str );
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 쥦ɥ (ɽ)
+* const char *str : 
+* long w : ɽ
+***************************************************************/
+
+void	call_gui_words_draw_width( const char *str, long w )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiWords.draw( str, w );
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 쥦ɥĤ
+***************************************************************/
+
+void	call_gui_words_wipe( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiWords.wipe();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* ץ쥤ɥ򳫻
+***************************************************************/
+
+void	call_gui_replay_bgn( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiReplay.bgn();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* ץ쥤ɥλ
+***************************************************************/
+
+void	call_gui_replay_end( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiReplay.end();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* ץ쥤ɥ
+***************************************************************/
+
+void	call_gui_replay_draw( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiReplay.draw();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* ץ쥤ɥĤ
+***************************************************************/
+
+void	call_gui_replay_wipe( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiReplay.wipe();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 饹ǥɥ
+***************************************************************/
+
+void	call_gui_class_edit_draw( class_t *p )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiClassEdit.draw( p );
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 饹ǥɥĤ
+***************************************************************/
+
+void	call_gui_class_edit_wipe( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiClassEdit.wipe();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 饯ȡɥ
+***************************************************************/
+
+void	call_gui_chr_sheet_draw( mbr_t *org, mbr_t *mbr )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiChrSheet.draw( org, mbr );
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 饯ȡɥ
+***************************************************************/
+
+void	call_gui_chr_sheet_redraw( mbr_t *org, mbr_t *mbr )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiChrSheet.redraw();
+
+	gui_end();
+#endif
+}
+
+/***************************************************************
+* 饯ȡɥĤ
+***************************************************************/
+
+void	call_gui_chr_sheet_wipe()
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gGuiChrSheet.wipe();
+
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -490,7 +592,7 @@ void	call_gui_input_string_wipe( void )
 * char *dir : եåǥ쥯ȥ
 ***************************************************************/
 
-void	set_str_dir_sel_graph( char *dir )
+void	call_set_str_dir_sel_graph( char *dir )
 {
 #if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	FileList::setStrDirSelGraph( dir );
@@ -569,84 +671,110 @@ bool_t	call_sel_conf_eval_set_str( long token, char *str )
 	return TRUE;
 }
 
+/***************************************************************
+* СΥեåѥ򳫻
+* mbr_t *mbr : С
+***************************************************************/
+
+bool_t	call_sel_mbr_graph_begin( mbr_t *mbr )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	bool_t flgWait = FALSE;
+
+	if( !g_flg_gui )
+		return FALSE;
+
+	gui_begin();
+
+	gSelMbrGraph = new SelMbrGraph;
+	flgWait = gSelMbrGraph->exec( mbr );
+
+	gui_end();
+
+	if( flgWait )
+		return TRUE;
+	else
+		return FALSE;
+#endif
+
+	return FALSE;
+}
+
+/***************************************************************
+* СΥեåѥλ
+***************************************************************/
+
+void	call_sel_mbr_graph_end( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	delete gSelMbrGraph;
+
+	gui_end();
+#endif
+}
+
 /***************************************************************
 * եåѥΥꥻå
 ***************************************************************/
 
 void	call_pcg_dun_reset( void )
 {
-#ifdef D_WS
-	if( !g_flg_init )
-		return;
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gPcgDun.reset();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
+	gui_end();
+#endif
+}
 
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
+/***************************************************************
+* СΥեåѥΥꥻå
+***************************************************************/
+
+void	call_pcg_dun_reload_all_mbr( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_gui )
 		return;
 
-	gdk_threads_enter();
+	gui_begin();
 
-	gPcgDun.reset();
+	gPcgDun.reloadAllMbr();
 
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	TRY {
-		gPcgDun.reset();
-	} CATCH( CException, e ){
-	} END_CATCH
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
 * Υޥåפ
 ***************************************************************/
 
-void	call_pcg_dun_draw_turn( void )
+void	call_pcg_dun_draw_turn( bool_t flg_first )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	if( flg_first )
+		gPcgDun.setFlgUpdateRequest( true );
+
+	gui_begin();
 
 	gPcgDun.drawTurn();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gPcgDun.drawTurn();
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+	gui_end();
+#endif
 
 #ifdef D_MFC
 	if( !g_flg_init )
@@ -654,8 +782,14 @@ void	call_pcg_dun_draw_turn( void )
 	if( !g_flg_gui )
 		return;
 
+	if( flg_first )
+		gPcgDun.setFlgUpdateRequest( true );
+
 	TRY {
-		gPcgDun.drawTurn();
+		if( g_Dir3d.CheckRefreshFrameMode() )
+			g_Dir3d.SetRefreshRequest( true );
+		else
+			gPcgDun.drawTurn();
 	} CATCH( CException, e ){
 	} END_CATCH
 #endif // D_MFC
@@ -673,32 +807,18 @@ void	call_pcg_dun_redraw( bool_t flgForce )
 	else
 		bFlagForce = false;
 
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gPcgDun.redraw( bFlagForce );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gPcgDun.redraw( bFlagForce );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+	gui_end();
+#endif
 
 #ifdef D_MFC
 	if( !g_flg_init )
@@ -707,7 +827,10 @@ void	call_pcg_dun_redraw( bool_t flgForce )
 		return;
 
 	TRY {
-		gPcgDun.redraw( bFlagForce );
+		if( g_Dir3d.CheckRefreshFrameMode() )
+			g_Dir3d.SetRefreshRequest( true );
+		else
+			gPcgDun.redraw( bFlagForce );
 	} CATCH( CException, e ){
 	} END_CATCH
 #endif // D_MFC
@@ -734,32 +857,18 @@ void	call_pcg_dun_draw(
 	else
 		bFlagForce = false;
 
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gPcgDun.draw( map_x, map_y, map_w, map_h, bFlagForce );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gPcgDun.draw( map_x, map_y, map_w, map_h, bFlagForce );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+	gui_end();
+#endif
 
 #ifdef D_MFC
 	if( !g_flg_init )
@@ -768,12 +877,49 @@ void	call_pcg_dun_draw(
 		return;
 
 	TRY {
-		gPcgDun.draw( map_x, map_y, map_w, map_h, bFlagForce );
+		if( g_Dir3d.CheckRefreshFrameMode() )
+			g_Dir3d.SetRefreshRequest( true );
+		else
+			gPcgDun.draw( map_x, map_y, map_w, map_h, bFlagForce );
 	} CATCH( CException, e ){
 	} END_CATCH
 #endif // D_MFC
 }
 
+/***************************************************************
+* ƥȡ⡼ɤλޥåפʸ
+* long scrn_x : X ɸ
+* long scrn_y : Y ɸ
+* const char *s : եޥåʸ
+* ... : եޥåȤΰ
+* return : 顼̵ä
+***************************************************************/
+
+void	call_pcg_dun_draw_string(
+	long scrn_x, long scrn_y,
+	const char *s, ... )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	va_list	argptr;
+	static char	buf[SCREEN_WIDTH * 2 + 1];
+
+	va_start( argptr, s );
+	vsn_printf( buf, SCREEN_WIDTH * 2, s, argptr );
+	va_end( argptr );
+
+	if( !g_flg_init )
+		return;
+	if( !g_flg_gui )
+		return;
+
+	gui_begin();
+
+	gPcgDun.drawString( scrn_x, scrn_y, MSG_S, buf );
+
+	gui_end();
+#endif
+}
+
 /***************************************************************
 * ޥåפΥ
 * long map_x : ޥåפ X ɸ
@@ -782,44 +928,18 @@ void	call_pcg_dun_draw(
 
 void	call_pcg_dun_scroll( long map_x, long map_y )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	gPcgDun.scrollTile( map_x, map_y );
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
-
-	gPcgDun.scrollTile( map_x, map_y );
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	TRY {
-		gPcgDun.scrollTile( map_x, map_y );
-	} CATCH( CException, e ){
-	} END_CATCH
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -830,38 +950,27 @@ void	call_pcg_dun_scroll( long map_x, long map_y )
 
 long	call_pcg_dun_get_chr_draw_left( chr_t *chr, bool_t flagPre )
 {
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	bool bFlagPre;
 	if( flagPre )
 		bFlagPre = true;
 	else
 		bFlagPre = false;
 
-#ifdef D_WS
+	if( g_flg_text_mode ){
+		if( flagPre )
+			return get_chr_left( chr->pre_draw_x, chr->dx );
+		else
+			return get_chr_left( chr->x, chr->dx );
+	}
+
 	Pcg *pPcg = gPcgDun.getChrPcg( chr );
 
 	long n = gPcgDun.getChrDrawLeft( chr, pPcg, bFlagPre );
 	n = n / gPcgDun.getTileSizeX();
 
 	return n;
-#endif // D_WS
-
-#ifdef D_GTK
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawLeft( chr, pPcg, bFlagPre );
-	n = n / gPcgDun.getTileSizeX();
-
-	return n;
-#endif // D_GTK
-
-#ifdef D_MFC
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawLeft( chr, pPcg, bFlagPre );
-	n = n / gPcgDun.getTileSizeX();
-
-	return n;
-#endif // D_MFC
+#endif
 
 	if( flagPre )
 		return get_chr_left( chr->pre_draw_x, chr->dx );
@@ -877,38 +986,27 @@ long	call_pcg_dun_get_chr_draw_left( chr_t *chr, bool_t flagPre )
 
 long	call_pcg_dun_get_chr_draw_right( chr_t *chr, bool_t flagPre )
 {
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	bool bFlagPre;
 	if( flagPre )
 		bFlagPre = true;
 	else
 		bFlagPre = false;
 
-#ifdef D_WS
+	if( g_flg_text_mode ){
+		if( flagPre )
+			return get_chr_right( chr->pre_draw_x, chr->dx );
+		else
+			return get_chr_right( chr->x, chr->dx );
+	}
+
 	Pcg *pPcg = gPcgDun.getChrPcg( chr );
 
 	long n = gPcgDun.getChrDrawRight( chr, pPcg, bFlagPre );
 	n = (n + gPcgDun.getTileSizeX() - 1) / gPcgDun.getTileSizeX();
 
 	return n;
-#endif // D_WS
-
-#ifdef D_GTK
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawRight( chr, pPcg, bFlagPre );
-	n = (n + gPcgDun.getTileSizeX() - 1) / gPcgDun.getTileSizeX();
-
-	return n;
-#endif // D_GTK
-
-#ifdef D_MFC
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawRight( chr, pPcg, bFlagPre );
-	n = (n + gPcgDun.getTileSizeX() - 1) / gPcgDun.getTileSizeX();
-
-	return n;
-#endif // D_MFC
+#endif
 
 	if( flagPre )
 		return get_chr_right( chr->pre_draw_x, chr->dx );
@@ -924,38 +1022,27 @@ long	call_pcg_dun_get_chr_draw_right( chr_t *chr, bool_t flagPre )
 
 long	call_pcg_dun_get_chr_draw_top( chr_t *chr, bool_t flagPre )
 {
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	bool bFlagPre;
 	if( flagPre )
 		bFlagPre = true;
 	else
 		bFlagPre = false;
 
-#ifdef D_WS
+	if( g_flg_text_mode ){
+		if( flagPre )
+			return get_chr_top( chr->pre_draw_y, chr->dy );
+		else
+			return get_chr_top( chr->y, chr->dy );
+	}
+
 	Pcg *pPcg = gPcgDun.getChrPcg( chr );
 
 	long n = gPcgDun.getChrDrawTop( chr, pPcg, bFlagPre );
 	n = n / gPcgDun.getTileSizeY();
 
 	return n;
-#endif // D_WS
-
-#ifdef D_GTK
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawTop( chr, pPcg, bFlagPre );
-	n = n / gPcgDun.getTileSizeY();
-
-	return n;
-#endif // D_GTK
-
-#ifdef D_MFC
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawTop( chr, pPcg, bFlagPre );
-	n = n / gPcgDun.getTileSizeY();
-
-	return n;
-#endif // D_MFC
+#endif
 
 	if( flagPre )
 		return get_chr_top( chr->pre_draw_y, chr->dy );
@@ -971,38 +1058,27 @@ long	call_pcg_dun_get_chr_draw_top( chr_t *chr, bool_t flagPre )
 
 long	call_pcg_dun_get_chr_draw_bottom( chr_t *chr, bool_t flagPre )
 {
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	bool bFlagPre;
 	if( flagPre )
 		bFlagPre = true;
 	else
 		bFlagPre = false;
 
-#ifdef D_WS
+	if( g_flg_text_mode ){
+		if( flagPre )
+			return get_chr_bottom( chr->pre_draw_y, chr->dy );
+		else
+			return get_chr_bottom( chr->y, chr->dy );
+	}
+
 	Pcg *pPcg = gPcgDun.getChrPcg( chr );
 
 	long n = gPcgDun.getChrDrawBottom( chr, pPcg, bFlagPre );
 	n = (n + gPcgDun.getTileSizeY() - 1) / gPcgDun.getTileSizeY();
 
 	return n;
-#endif // D_WS
-
-#ifdef D_GTK
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawBottom( chr, pPcg, bFlagPre );
-	n = (n + gPcgDun.getTileSizeY() - 1) / gPcgDun.getTileSizeY();
-
-	return n;
-#endif // D_GTK
-
-#ifdef D_MFC
-	Pcg *pPcg = gPcgDun.getChrPcg( chr );
-
-	long n = gPcgDun.getChrDrawBottom( chr, pPcg, bFlagPre );
-	n = (n + gPcgDun.getTileSizeY() - 1) / gPcgDun.getTileSizeY();
-
-	return n;
-#endif // D_MFC
+#endif
 
 	if( flagPre )
 		return get_chr_bottom( chr->pre_draw_y, chr->dy );
@@ -1019,15 +1095,11 @@ long	call_pcg_dun_get_chr_draw_bottom( chr_t *chr, bool_t flagPre )
 
 bool_t	call_pcg_dun_clip_win( long mapX, long mapY )
 {
-#ifdef D_GTK
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	return gPcgDun.clipWin( mapX, mapY );
-#endif // D_GTK
+#endif
 
-#ifdef D_MFC
-	return gPcgDun.clipWin( mapX, mapY );
-#endif // D_MFC
-
-	return true;
+	return TRUE;
 }
 
 /***************************************************************
@@ -1038,15 +1110,11 @@ bool_t	call_pcg_dun_clip_win( long mapX, long mapY )
 
 bool_t	call_pcg_dun_clip_win_x( long mapX )
 {
-#ifdef D_GTK
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	return gPcgDun.clipWinX( mapX );
-#endif // D_GTK
+#endif
 
-#ifdef D_MFC
-	return gPcgDun.clipWinX( mapX );
-#endif // D_MFC
-
-	return true;
+	return TRUE;
 }
 
 /***************************************************************
@@ -1057,15 +1125,141 @@ bool_t	call_pcg_dun_clip_win_x( long mapX )
 
 bool_t	call_pcg_dun_clip_win_y( long mapY )
 {
-#ifdef D_GTK
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	return gPcgDun.clipWinY( mapY );
-#endif // D_GTK
+#endif
 
-#ifdef D_MFC
-	return gPcgDun.clipWinY( mapY );
-#endif // D_MFC
+	return TRUE;
+}
 
-	return true;
+/***************************************************************
+* 饹ܥΥǥ⤬äĴ٤
+* return : ǥ⤬ä?
+***************************************************************/
+
+void	call_demo_last_boss_reset( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( gPcgDun.pDemoLastBoss == NULL )
+		return;
+
+	gPcgDun.pDemoLastBoss->reset();
+#endif
+}
+
+/***************************************************************
+* 饹ܥΥǥ⤬äĴ٤
+* return : ǥ⤬ä?
+***************************************************************/
+
+bool_t	call_demo_last_boss_check_done( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( gPcgDun.pDemoLastBoss == NULL )
+		return TRUE;
+
+	if( gPcgDun.pDemoLastBoss->checkDone() )
+		return TRUE;
+	else
+		return FALSE;
+#endif
+
+	return TRUE;
+}
+
+/***************************************************************
+* ˹礻 BGM κ
+***************************************************************/
+
+void	call_game_music_play_scene( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	gMusic.playScene();
+#endif
+}
+
+/***************************************************************
+* BGM κ
+* music_kind_t kind : BGM μ
+* long idx : BGM ꥹȤΥǥå
+***************************************************************/
+
+void	call_game_music_play( long kind, long idx )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	gMusic.play( (music_kind_t)kind, idx );
+#endif
+}
+
+/***************************************************************
+* Υ祤ƥåֹ֤
+* return : 祤ƥåֹ
+***************************************************************/
+
+long	call_game_joystick_get_joy( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	return gJoystick.getJoy();
+#endif
+
+	return g_joy_n;
+}
+
+/***************************************************************
+* 祤ƥå
+* long n : 祤ƥåֹ
+***************************************************************/
+
+void	call_game_joystick_sel_joy( long n )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	gJoystick.selJoy( n );
+#endif
+
+	g_joy_n = n;
+}
+
+/***************************************************************
+* 祤ƥå֤̾
+* long n : 祤ƥåֹ
+* return : 祤ƥå̾
+***************************************************************/
+
+const char	*call_game_joystick_get_name( long n )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	return gJoystick.getName( n );
+#endif
+
+	return NULL;
+}
+
+/***************************************************************
+* 祤ƥåμ֤
+* return : 祤ƥåμ
+***************************************************************/
+
+long	call_game_joystick_get_kind( void )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	return gJoystick.getKind();
+#endif
+
+	return g_joy_kind;
+}
+
+/***************************************************************
+* 祤ƥåμ
+* joy_kind_t kind : 祤ƥåμ
+***************************************************************/
+
+void	call_game_joystick_set_kind( long kind )
+{
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	gJoystick.setKind( (joy_kind_t)kind );
+#endif
+
+	g_joy_kind = (joy_kind_t)kind;
 }
 
 /***************************************************************
@@ -1074,41 +1268,18 @@ bool_t	call_pcg_dun_clip_win_y( long mapY )
 
 void	call_gui_stat_redraw( void )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
-
-	StatWin->redraw();
-
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
+	gui_begin();
 
 	gGuiStat.redraw();
 
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gGuiStat.redraw();
-#endif // D_MFC
+	gui_end();
+#endif
 }
 
 /***************************************************************
@@ -1118,41 +1289,112 @@ void	call_gui_stat_redraw( void )
 
 void	call_gui_stat_draw( long n )
 {
-#ifdef D_WS
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	if( !g_flg_init )
 		return;
 	if( !g_flg_gui )
 		return;
 
-	WSGIappDev()->lock();
-
-	StatWin->draw( n );
-
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return;
-	if( !g_flg_gui )
-		return;
-
-	gdk_threads_enter();
+	gui_begin();
 
 	gGuiStat.draw( n );
 
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
+	gui_end();
+#endif
+}
 
-#ifdef D_MFC
-	if( !g_flg_init )
+/***************************************************************
+* ɥκɸ
+* win_kind_t k : ɥμ
+* long nx : X ɸ
+* long ny : Y ɸ
+* long nw : 
+* long nh : ⤵
+***************************************************************/
+
+void	call_win_pos_set(
+	win_kind_t k,
+	long *px, long *py,
+	long *pw, long *ph
+)
+{
+	if( k < WIN_KIND_NULL )
 		return;
-	if( !g_flg_gui )
+	if( k >= WIN_KIND_MAX_N )
 		return;
 
-	gGuiStat.draw( n );
-#endif // D_MFC
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( g_flg_gui ){
+		gui_begin();
+
+		gWinPos[k].set( k, px, py, pw, ph );
+
+		gui_end();
+
+		return;
+	}
+#endif
+
+	if( px != NULL )
+		g_win_x[k] = *px;
+	if( py != NULL )
+		g_win_y[k] = *py;
+	if( pw != NULL )
+		g_win_w[k] = *pw;
+	if( ph != NULL )
+		g_win_h[k] = *ph;
+}
+
+/***************************************************************
+* ɥκɸμ
+* win_kind_t k : ɥμ
+* long *px : X ɸ
+* long *py : Y ɸ
+* long *pw : 
+* long *ph : ⤵
+* return : Ǥ?
+***************************************************************/
+
+bool_t	call_win_pos_get(
+	win_kind_t k,
+	long *px, long *py,
+	long *pw, long *ph
+)
+{
+	if( k < WIN_KIND_NULL )
+		return FALSE;
+	if( k >= WIN_KIND_MAX_N )
+		return FALSE;
+
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
+	if( g_flg_gui ){
+		gui_begin();
+
+		gWinPos[k].get( k, px, py, pw, ph );
+
+		gui_end();
+
+		return TRUE;
+	}
+#endif
+
+	if( px != NULL )
+		*px = g_win_x[k];
+	if( py != NULL )
+		*py = g_win_y[k];
+	if( pw != NULL )
+		*pw = g_win_w[k];
+	if( ph != NULL )
+		*ph = g_win_h[k];
+
+	if( pw != NULL )
+		if( *pw <= 0 )
+			*pw = 256;
+	if( ph != NULL )
+		if( *ph <= 0 )
+			*ph = 256;
+
+	return TRUE;
 }
 
 /***************************************************************
@@ -1162,43 +1404,34 @@ void	call_gui_stat_draw( long n )
 
 long	call_key_get_char( void )
 {
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
 	long	c = '\0';
 
-#ifdef D_WS
 	if( !g_flg_init )
 		return c;
 	if( !g_flg_gui )
 		return c;
 
-	WSGIappDev()->lock();
+	gui_begin();
 
 	c = gKey.getChar();
 
-	WSGIappDev()->unlock();
-#endif // D_WS
-
-#ifdef D_GTK
-	if( !g_flg_init )
-		return c;
-	if( !g_flg_gui )
-		return c;
-
-	gdk_threads_enter();
-
-	c = gKey.getChar();
-
-	gdk_flush();
-	gdk_threads_leave();
-#endif // D_GTK
-
-#ifdef D_MFC
-	if( !g_flg_init )
-		return c;
-	if( !g_flg_gui )
-		return c;
-
-	c = gKey.getChar();
-#endif // D_MFC
+	gui_end();
 
 	return c;
+#endif
+
+	return '\0';
+}
+
+/***************************************************************
+* ϤΥ
+* long n :  (ߥ)
+***************************************************************/
+
+void	call_sleep( long n )
+{
+#ifdef	D_MFC
+	::Sleep( n );
+#endif	/* D_MFC */
 }
