@@ -7,6 +7,7 @@ extern "C" {
 void	init_chr( void );
 void	reset_static_chr( void );
 void	exec_all_phase( void );
+void	last_boss_phase( void );
 void	draw_phase( bool_t flg_first_phase );
 void	move_phase_chr( chr_t *chr );
 void	act_phase_chr( chr_t *chr );
@@ -23,9 +24,16 @@ void	set_map_chr_dun( chr_t *chr );
 void	set_clr_map_chr_dun(
 	chr_t *chr, char mjr, char mnr, flg_map_t flg
 );
+void	set_map_chr_hyper_boss( chr_t *chr );
 void	move_chr( chr_t *chr );
 void	move_chr_sub( chr_t *chr );
 void	move_chr_std( chr_t *chr );
+void	move_chr_mbr( chr_t *chr );
+bool_t	chk_move_chr_dir( chr_t *chr, long *nx, long *ny );
+bool_t	chk_move_chr_far_axis( chr_t *chr, long *nx, long *ny );
+bool_t	chk_move_chr_cross_axis( chr_t *chr, long *nx, long *ny );
+bool_t	chk_move_chr_scan_x( chr_t *chr, long sx, long *lx );
+bool_t	chk_move_chr_scan_y( chr_t *chr, long sy, long *ly );
 void	chk_chr_force_move( chr_t *chr );
 bool_t	chk_chr_stagger( chr_t *chr );
 bool_t	chk_chr_stagger_sub( chr_t *chr, flg_stat_t flg, long rate );
@@ -135,6 +143,7 @@ bool_t	chk_resi_stat( chr_t *chr, flg_stat_t stat );
 bool_t	chk_resi_stat_or( chr_t *chr, flg_stat_t stat );
 rate_t	set_move_rate( chr_t *chr );
 rate_t	get_move_rate( chr_t *chr );
+const char	*get_sex_name( sex_t sex );
 char	*get_abl_name( abl_kind_t kind );
 abl_kind_t	get_sub_abl( abl_kind_t abl_main );
 long	calc_lev_to_exp( long lev, abl_kind_t abl );
@@ -143,6 +152,7 @@ long	get_exp( chr_t *chr, abl_kind_t abl );
 long	get_lev( chr_t *chr, abl_kind_t abl );
 long	get_lev_max( chr_t *chr, abl_kind_t abl );
 rate_t	get_resi( chr_t *chr, resi_kind_t resi );
+const char	*get_resi_name( resi_kind_t kind );
 bool_t	chk_modified_lev( chr_t *chr, abl_kind_t abl );
 long	calc_abl_to_val( chr_t *chr, abl_kind_t abl );
 long	calc_lev_to_val( long lev, abl_kind_t abl );
