@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: curs.c,v 1.76 2008/04/30 23:44:50 bowkenken Exp $
+* $Id: curs.c,v 1.79 2008/09/07 11:31:27 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -64,6 +64,9 @@ static curs_attr_t	g_curs_attr[CURS_ATTR_MAX_N] = {
 	{ 0, N_MSG_ATTR_ERR,
 		COLOR_RED, COLOR_BLACK, A_BOLD, },
 
+	{ 0, N_MSG_ATTR_MENU_NORMAL,
+		COLOR_WHITE, COLOR_BLACK, A_NORMAL, },
+
 	{ 0, N_MSG_ATTR_MENU_DIS,
 		COLOR_YELLOW, COLOR_BLACK, A_NORMAL, },
 
@@ -170,8 +173,8 @@ bool_t	init_curs_color( void )
 		return TRUE;
 	}
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
-# if	HAVE_HAS_COLORS
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
+# if	defined( HAVE_HAS_COLORS )
 	if( g_flg_cui && !has_colors() ){
 		flg_color = FALSE;
 		return FALSE;
@@ -181,7 +184,7 @@ bool_t	init_curs_color( void )
 	return FALSE;
 # endif	/* HAVE_HAS_COLORS */
 
-# if	HAVE_START_COLOR
+# if	defined( HAVE_START_COLOR )
 	if( g_flg_cui && (start_color() != OK) ){
 		flg_color = FALSE;
 		return FALSE;
@@ -212,11 +215,11 @@ bool_t	init_curs_color( void )
 
 bool_t	init_curs_color_pair( curs_attr_t *p, short pair_n )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( !flg_color )
 		return FALSE;
 
-# if	HAVE_INIT_PAIR
+# ifdef	HAVE_INIT_PAIR
 	if( g_flg_cui )
 		init_pair( pair_n, p->fg, p->bg );
 # endif	/* HAVE_INIT_PAIR */
@@ -322,7 +325,7 @@ void	curs_clearok( WINDOW *win, bool_t bf )
 {
 	flg_curs++;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( flg_curs == 1 ){
 		if( chk_curs() ){
 			if( g_flg_cui ){
@@ -342,7 +345,7 @@ void	curs_initscr( void )
 {
 	flg_curs++;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( flg_curs == 1 ){
 		if( chk_curs() ){
 			if( g_flg_cui ){
@@ -363,7 +366,7 @@ void	curs_endwin( void )
 {
 	if( flg_curs == 1 ){
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 		if( chk_curs() ){
 			dis_curs();
 			if( g_flg_cui ){
@@ -389,10 +392,10 @@ void	curs_endwin( void )
 
 void	curs_keypad( WINDOW *win, bool_t bf )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( g_flg_use_keypad ){
 		if( chk_curs() ){
-# if	HAVE_KEYPAD
+# ifdef	HAVE_KEYPAD
 			if( g_flg_cui ){
 				keypad( win, bf );
 			}
@@ -411,7 +414,7 @@ void	curs_refresh( void )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			refresh();
@@ -488,9 +491,9 @@ void	curs_attrset( long at )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
-# if	HAVE_ATTRSET
+# ifdef	HAVE_ATTRSET
 		if( g_flg_cui ){
 			attrset( at );
 		}
@@ -531,9 +534,9 @@ void	curs_attron( long at )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
-# if	HAVE_ATTRON
+# ifdef	HAVE_ATTRON
 		if( g_flg_cui ){
 			attron( at );
 		}
@@ -560,9 +563,9 @@ void	curs_attroff( long at )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
-# if	HAVE_ATTROFF
+# ifdef	HAVE_ATTROFF
 		if( g_flg_cui ){
 			attroff( at );
 		}
@@ -595,7 +598,7 @@ curs_attr_t	*get_curs_attr( void )
 
 void	curs_echo( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			echo();
@@ -610,7 +613,7 @@ void	curs_echo( void )
 
 void	curs_noecho( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			noecho();
@@ -625,7 +628,7 @@ void	curs_noecho( void )
 
 void	curs_cbreak( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			cbreak();
@@ -640,7 +643,7 @@ void	curs_cbreak( void )
 
 void	curs_nocbreak( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			nocbreak();
@@ -655,7 +658,7 @@ void	curs_nocbreak( void )
 
 void	curs_raw( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			raw();
@@ -670,7 +673,7 @@ void	curs_raw( void )
 
 void	curs_noraw( void )
 {
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			noraw();
@@ -701,7 +704,7 @@ void	curs_move( int scrn_y, int scrn_x )
 		return;
 #endif	/* ! D_DOS */
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			move( scrn_y, scrn_x );
@@ -743,7 +746,7 @@ void	curs_printw( char *s, ... )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		va_start( argptr, s );
 		vwprintw( stdscr, s, argptr );
@@ -767,7 +770,7 @@ void	curs_clear( void )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		if( g_flg_cui ){
 			curs_attrset_dflt();
@@ -811,7 +814,7 @@ int	curs_getch( void )
 
 	c = '\0';
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 
 	if( !flg_getc ){
 		if( chk_curs() ){
@@ -859,9 +862,9 @@ void	curs_getnstr( char *s, int len )
 	if( !chk_curs_draw() )
 		return;
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
-# if	HAVE_GETNSTR
+# ifdef	HAVE_GETNSTR
 		getnstr( s, len );
 		return;
 # endif	/* HAVE_GETNSTR */
@@ -895,11 +898,11 @@ long	get_max_line( void )
 	if( !chk_flg_screen_size_auto() )
 		return get_screen_size_lines();
 
-#if	HAVE_NCURSES_H || HAVE_CURSES_H
+#if	defined( HAVE_NCURSES_H ) || defined( HAVE_CURSES_H )
 	if( chk_curs() ){
 		return LINES;
 	}
-#endif	/* HAVE_NCURSES_H || HAVE_CURSES_H */
+#endif
 
 #ifdef	D_DOS
 	{
@@ -963,6 +966,32 @@ void	set_screen_size_lines( long n )
 	g_screen_size_lines = n;
 }
 
+/***************************************************************
+* ⡼ɤ
+* curs_attr_t *a : ⡼
+* curs_attr_t *b : ⡼
+* return : ⡼ɤƱ?
+***************************************************************/
+
+bool_t	cmp_attr( curs_attr_t *a, curs_attr_t *b )
+{
+	if( a == NULL )
+		return FALSE;
+	if( b == NULL )
+		return FALSE;
+
+	if( a->color_pair_n != b->color_pair_n )
+		return FALSE;
+	if( a->fg != b->fg )
+		return FALSE;
+	if( a->bg != b->bg )
+		return FALSE;
+	if( a->attr != b->attr )
+		return FALSE;
+
+	return TRUE;
+}
+
 /***************************************************************
 * DOS ѤοѴ
 * curs_attr_t *attr : ⡼
