@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: curs.h,v 1.35 2008/04/30 21:30:50 bowkenken Exp $
+* $Id: curs.h,v 1.37 2008/09/07 11:31:27 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	CURS_H
@@ -38,19 +38,19 @@
 #include	"extern.h"
 #include	"msg.h"
 
-#if	HAVE_NCURSES_H
+#if	defined( HAVE_NCURSES_H )
 # include	<ncurses.h>
 # ifndef	COLOR_WHITE
 #  include	"curs-def.h"
-# endif	/* COLOR_WHITE */
-#elif	HAVE_CURSES_H
+# endif
+#elif	defined( HAVE_CURSES_H )
 # include	<curses.h>
 # ifndef	COLOR_WHITE
 #  include	"curs-def.h"
-# endif	/* COLOR_WHITE */
-#else	/* HAVE_LIBCURSES */
+# endif
+#else
 # include	"curs-def.h"
-#endif	/* HAVE_LIBCURSES */
+#endif
 
 /***************************************************************
 * 
@@ -70,6 +70,7 @@ typedef enum {
 	CURS_ATTR_N_NORMAL,
 	CURS_ATTR_N_REV,
 	CURS_ATTR_N_ERR,
+	CURS_ATTR_N_MENU_NORMAL,
 	CURS_ATTR_N_MENU_DIS,
 	CURS_ATTR_N_MENU_SEL,
 	CURS_ATTR_N_MAP_ITEM,
