@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: dun-tab.h,v 1.8 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: dun-tab.h,v 1.14 2008/10/07 10:27:00 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	DUN_TAB_H
@@ -878,4 +878,173 @@ static room_ptn_t	boss_room_ptn[] = {
 	},
 };
 
+/***************************************************************
+* 饹ܥΥޥåסǡ
+***************************************************************/
+
+static room_ptn_t	last_boss_room_ptn[LAST_BOSS_ROOM_PTN_MAX_N] = {
+	{
+		DUN_LEV_LAST_BOSS, DUN_LEV_LAST_BOSS, -1, _100_PERCENT,
+		{
+			".0####################################. ",
+			"####        . . . . . . .           ####",
+			"##        . . . ######. . .           ##",
+			"##        . . . ##XX##. . .           ##",
+			"##        . . . . . . . . .           ##",
+			"##        . . XL. . . XR. .           ##",
+			"##          . . . . . . .             ##",
+			"##                                    ##",
+			"##                                    ##",
+			"##                                    ##",
+			"##                                    ##",
+			"##                                    ##",
+			"##                                    ##",
+			"##    . . . . . . . . . . . . . .     ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  .9.9.9.9.9.9.9.9.9.9.9.9.9.9.9.9  ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . .2. . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+		},
+		{
+			"bbbbbbbbbbbbbbbbbbbb",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+		}
+	},
+	{
+		DUN_LEV_LAST_BOSS, DUN_LEV_LAST_BOSS, -1, _100_PERCENT,
+		{
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##  . . . . . . . . . . . . . . . .   ##",
+			"##    . . . . . . . . . . . . . .     ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+			"##              . . . .               ##",
+		},
+		{
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+		}
+	},
+	{
+		DUN_LEV_LAST_BOSS, DUN_LEV_LAST_BOSS, -1, _100_PERCENT,
+		{
+			"################+ + + + ################",
+			"################+ + + + ################",
+			"##. . . .b####.b. . . . .c####.c. . . ##",
+			"##. . . .b####.b. . . . .c####.c. . . ##",
+			"##. . . .b.b.b.b. . . . .c.c.c.c. . . ##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##.a.a.a. . . . . . . . . . . . .d.d.d##",
+			"######.a. . . . . .1. . . . . . .d######",
+			"######.a. . . . . . . . . . . . .d######",
+			"##.a.a.a. . . . . . . . . . . . .d.d.d##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##. . . . . . . . . . . . . . . . . . ##",
+			"##. . . .f.f.f.f. . . . .e.e.e.e. . . ##",
+			"##. . . .f####.f. . . . .e####.e. . . ##",
+			"##. . . .f####.f. . . . .e####.e. . . ##",
+			"########################################",
+			"########################################",
+		},
+		{
+			"bbbbbbbbbbbbbbbbbbbb",
+			"bbbbbbbbbbbbbbbbbbbb",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"baaaaaaaaaaaaaaaaaab",
+			"bbbbbbbbbbbbbbbbbbbb",
+			"bbbbbbbbbbbbbbbbbbbb",
+		}
+	}
+};
+
+/* ޥåס٥ */
+
+map_event_tab_t	last_boss_map_event_tab[] = {
+	/* ݥ */
+	{ 'a', NULL, },
+	{ 'b', NULL, },
+	{ 'c', NULL, },
+	{ 'd', NULL, },
+	{ 'e', NULL, },
+	{ 'f', NULL, },
+
+	/* ѡƥи */
+	{ MAP_EVENT_LAST_BOSS_APPEAR_PARTY, NULL, },
+	/* Ʈϰ */
+	{ MAP_EVENT_LAST_BOSS_BATTLE, NULL, },
+	/* 饹ܥ٥ȳϰ */
+	{ MAP_EVENT_LAST_BOSS_WORDS, begin_last_boss_event, },
+
+	{ '\0', NULL, },
+};
+
 #endif	/* DUN_TAB_H */
