@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: dun.c,v 1.117 2008/09/06 14:49:19 bowkenken Exp $
+* $Id: dun.c,v 1.129 2008/10/07 10:27:00 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -43,28 +43,6 @@
 /* ϩɽꥢμ */
 #define	AREA_PATH	-1
 
-/* Ψ */
-
-/* ꥢˤʤΨ */
-#define	RATE_ROOM	((rate_t)50)
-
-/* ɥƤΨ */
-#define	RATE_DOOR_OPEN	((rate_t)10)
-
-/* ɥꤵƤΨ */
-#define	RATE_DOOR_JAMMED	((rate_t)5)
-
-/* ɥƤΨ */
-#define	RATE_DOOR_BROKEN	((rate_t)1)
-
-/* ϩƤΨ */
-#define	RATE_HIDE_CROSS_PATH	((rate_t)80)
-
-/* 󥹥ΥߥߥåȢǤʤΨ */
-#define	MIMIC_RATE	50
-
-/***************************************************************/
-
 /* 곬ʤο */
 #define	STAIRS_UP_MAX_N	2
 
@@ -83,11 +61,41 @@
 /* 1ֶᤤɥõƱΥˤȽǤΥ */
 #define	DOOR_NEAR_R	4
 
+#define	LAST_BOSS_ROOM_PTN_MAX_N	3
+
+/***************************************************************
+* Ψ
+***************************************************************/
+
+/* ꥢ꤬ƤγΨ */
+#define	LIGHTED_ROOM_MAX_RATE	((rate_t)80)
+
+/* ꥢˤʤΨ */
+#define	RATE_ROOM	((rate_t)50)
+
+/* ɥƤΨ */
+#define	RATE_DOOR_OPEN	((rate_t)10)
+
+/* ɥꤵƤΨ */
+#define	RATE_DOOR_JAMMED	((rate_t)5)
+
+/* ɥƤΨ */
+#define	RATE_DOOR_BROKEN	((rate_t)1)
+
+/* ϩƤΨ */
+#define	RATE_HIDE_CROSS_PATH	((rate_t)80)
+
+/* 󥹥ΥߥߥåȢǤʤΨ */
+#define	MIMIC_RATE	((rate_t)50)
+
 /***************************************************************
 * ѿ
 ***************************************************************/
 
 static dun_t	dun;
+
+static long	g_last_boss_lev;
+
 static area_t	g_room_ptn_max_n, g_boss_room_ptn_max_n;
 
 static crsr_ptn_t	g_crsr_ptn, g_crsr_ptn_usr[CRSR_PTN_MAX_N];
@@ -114,6 +122,13 @@ void	init_dun( void )
 	g_nest_flg_dun = make_nest_flg();
 	bgn_nest_flg( g_nest_flg_dun );
 
+	g_last_boss_lev = -DUN_LEV_LOOP;
+
+	g_room_ptn_max_n = sizeof( room_ptn )
+			/ sizeof( room_ptn_t );
+	g_boss_room_ptn_max_n = sizeof( boss_room_ptn )
+			/ sizeof( room_ptn_t );
+
 	reset_dun( FALSE );
 
 	/* cursor */
@@ -261,6 +276,8 @@ void	reset_dun( bool_t flg_wall )
 
 	dun.door_n = 0;
 
+	reset_map_event();
+
 	end_nest_flg( g_nest_flg_dun );
 }
 
@@ -352,6 +369,8 @@ void	up_dun( long d )
 	if( dun.lev < -DUN_MAX_LEV_BASE )
 		dun.lev = -DUN_MAX_LEV_BASE;
 
+	set_last_boss_lev( dun.lev );
+
 	if( g_flg_init ){
 		if( get_flg_no_save() )
 			menu_save_n( TRUE, SAVE_N_AUTO );
@@ -372,7 +391,10 @@ void	up_dun( long d )
 		change_scene( SCENE_N_TOWN );
 	}
 
+	/* Сãؤ򹹿 */
+
 	for( i = 0; i < MBR_MAX_N; i++ ){
+		/* Ͼ */
 		if( sgn_l( dun.lev ) >= +1 ){
 			if( pty->mbr[i]->dun_max_lev_floor
 					< labs( dun.lev ) ){
@@ -380,6 +402,7 @@ void	up_dun( long d )
 						= labs( dun.lev );
 			}
 		}
+		/* ϲ */
 		if( sgn_l( dun.lev ) <= -1 ){
 			if( pty->mbr[i]->dun_max_lev_base
 					< labs( dun.lev ) ){
@@ -392,6 +415,22 @@ void	up_dun( long d )
 	reset_all( TRUE );
 }
 
+/***************************************************************
+* 饹ܥγؤ
+***************************************************************/
+
+void	enter_last_boss_dun( void )
+{
+	change_last_scene( LAST_SCENE_N_ENTER );
+
+	if( dun.lev != DUN_LEV_LAST_BOSS )
+		set_last_boss_lev( dun.lev );
+	dun.lev = DUN_LEV_LAST_BOSS;
+
+	reset_all( TRUE );
+	set_map_event_tab( last_boss_map_event_tab );
+}
+
 /***************************************************************
 * µܤ(μ)
 * bool_t flg_appear : ѡƥνи򤹤뤫?
@@ -427,7 +466,10 @@ void	reset_all( bool_t flg_appear )
 	clr_map_all_mbr();
 	clr_map_all_pet();
 
-	if( dun.lev == 0 ){
+	if( dun.lev == DUN_LEV_LAST_BOSS ){
+		make_dun_last_boss();
+		appear_party_last_boss();
+	} else if( dun.lev == 0 ){
 		make_town();
 		if( flg_appear ){
 			if( !appear_party_town() )
@@ -443,8 +485,7 @@ void	reset_all( bool_t flg_appear )
 
 	end_nest_flg( g_nest_flg_dun );
 
-	if( g_flg_gui )
-		call_pcg_dun_reset();
+	call_pcg_dun_reset();
 	redraw_all();
 }
 
@@ -464,20 +505,9 @@ void	make_dun( void )
 
 	bgn_nest_flg( g_nest_flg_dun );
 
-	reset_dun( TRUE );
-
-	g_room_ptn_max_n = sizeof( room_ptn )
-			/ sizeof( room_ptn_t );
-	g_boss_room_ptn_max_n = sizeof( boss_room_ptn )
-			/ sizeof( room_ptn_t );
-
 	/*  */
 
-	for( y = 0; y < AREA_MAP_MAX_Y; y++ ){
-		for( x = 0; x < AREA_MAP_MAX_X; x++ ){
-			dun.area[y][x] = AREA_PATH;
-		}
-	}
+	reset_dun( TRUE );
 
 	x = randm( AREA_MAP_MAX_X );
 	y = randm( AREA_MAP_MAX_Y );
@@ -551,6 +581,11 @@ void	make_dun( void )
 		}
 	}
 
+	/* 饹ܥؤγʤ¤ */
+
+	if( labs( dun.lev ) >= labs( DUN_LEV_LAST_BOSS_STAIRS ) )
+		make_stairs_randm( FACE_MNR_STAIRS_LAST_BOSS );
+
 	/* ꥢƥ */
 
 	item_n = 0;
@@ -709,8 +744,12 @@ void	make_dun( void )
 		for( j = 0; j < LOOP_MAX_100; j++ ){
 			x = 1 + randm( MAP_MAX_X - 2 );
 			y = 1 + randm( MAP_MAX_Y - 2 );
-			if( dun.map.obj.mjr[y][x] == FACE_MJR_FLOOR )
-				break;
+			if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
+				continue;
+			if( dun.map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+				continue;
+
+			break;
 		}
 
 		trap = make_trap( x, y, dun.lev );
@@ -734,6 +773,80 @@ void	make_dun( void )
 	end_nest_flg( g_nest_flg_dun );
 }
 
+/***************************************************************
+* 饹ܥΥ٥γؤ
+* long dun_lev : 
+***************************************************************/
+
+void	set_last_boss_lev( long dun_lev )
+{
+	if( dun_lev == 0 )
+		g_last_boss_lev = -DUN_LEV_LOOP;
+	else
+		g_last_boss_lev = dun_lev;
+}
+
+/***************************************************************
+* 饹ܥγؤ
+***************************************************************/
+
+void	make_dun_last_boss( void )
+{
+	long	x, y;
+	long	i;
+
+	bgn_nest_flg( g_nest_flg_dun );
+
+	/*  */
+
+	reset_dun( TRUE );
+
+	/* ¤ */
+
+	x = AREA_MAP_MAX_X / 2;
+	y = 0;
+	for( i = 0; i < LAST_BOSS_ROOM_PTN_MAX_N; i++ ){
+		y = 0 + i;
+		make_area_last_boss( &(last_boss_room_ptn[i]),
+				(area_t)1, x, y );
+	}
+
+	/* ޥåפüɤˤ */
+
+	make_fence();
+
+	/* 󥹥 */
+
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
+			mnstr_kind_t	mnstr_kind;
+			char	mjr, mnr;
+
+			mjr = dun.map.obj.mjr[y][x];
+			mnr = dun.map.obj.mnr[y][x];
+
+			/* GOD ֤ʤ */
+			if( !isalpha( mjr ) )
+				continue;
+
+			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
+			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
+			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+
+			mnstr_kind = get_mnstr_kind_from_face( mjr, mnr );
+			if( mnstr_kind == MNSTR_KIND_NULL )
+				continue;
+
+			make_mnstr( x, y, TRUE, g_last_boss_lev,
+					mnstr_kind );
+		}
+	}
+
+	/**/
+
+	end_nest_flg( g_nest_flg_dun );
+}
+
 /***************************************************************
 * µܤγȤɤ
 ***************************************************************/
@@ -840,6 +953,24 @@ bool_t	make_area_boss( room_ptn_t *ptn, area_t max_n, long x, long y )
 	return TRUE;
 }
 
+/***************************************************************
+* µܤΥ饹ܥ
+* room_ptn_t *ptn : Υѥ
+* area_t max_n : Υѥκ
+* long x : X ɸ
+* long y : Y ɸ
+***************************************************************/
+
+bool_t	make_area_last_boss( room_ptn_t *ptn, area_t max_n, long x, long y )
+{
+	if( dun.area[y][x] != AREA_PATH )
+		return FALSE;
+
+	set_room( ptn, (area_t)0, x, y, FLG_NULL );
+
+	return TRUE;
+}
+
 /***************************************************************
 * µܤγؤܥФ뤫å
 * long dun_lev : µܤγ
@@ -887,29 +1018,55 @@ void	set_room( room_ptn_t *room, area_t n, long x, long y, long flg_rev )
 			mx = x * AREA_MAX_X + jj;
 			my = y * AREA_MAX_Y + ii;
 
+			/* Υѥɤ߹ */
+
 			mjr = room[n].ptn[i][j * 2 + 0];
 			mnr = room[n].ptn[i][j * 2 + 1];
 			sect = room[n].sect[i][j];
 
+			/* Ϸˤ */
+
+			if( mjr == FACE_MJR_NULL ){
+				mjr = FACE_MJR_FLOOR;
+				mnr = FACE_MNR_HOLE;
+			}
+
+			/* Υѥ񤭹 */
+
 			dun.map.obj.mjr[my][mx] = mjr;
 			dun.map.obj.mnr[my][mx] = mnr;
 			dun.map.obj.flg[my][mx] = FLG_NULL;
 			dun.map.sect[my][mx] = sect;
 
+			/* ե饰񤭹 */
+
+			flg = FLG_MAP_OBJ_PASS;
 			switch( mjr ){
 			case FACE_MJR_FLOOR:
+				if( mnr == FACE_MNR_HOLE )
+					flg &= ~(FLG_MAP_OBJ_PASS);
+				else
+					flg |= FLG_MAP_OBJ_PASS;
+				break;
 			case FACE_MJR_DOOR_OPEN:
-				flg = FLG_MAP_OBJ_PASS;
+				flg |= FLG_MAP_OBJ_PASS;
 				break;
 			case FACE_MJR_WALL:
 			case FACE_MJR_DOOR_CLOSE:
-				flg = FLG_NULL;
+				flg &= ~(FLG_MAP_OBJ_PASS);
 				break;
 			default:
-				flg = FLG_MAP_OBJ_PASS;
+				flg |= FLG_MAP_OBJ_PASS;
 			}
 			dun.map.obj.flg[my][mx] = flg;
 
+			/* ޥåס٥Ȥκɸ */
+
+			if( (mjr == FACE_MJR_FLOOR) && isalnum( mnr ) )
+				set_map_event_pos( mnr, mx, my );
+
+			/*  */
+
 			if( flg_light )
 				dun.map.light_depth_obj[my][mx] = 1;
 			else
@@ -943,7 +1100,10 @@ bool_t	chk_lighted_room( void )
 {
 	rate_t	rate;
 
-	rate = 80 - (randm( labs( dun.lev ) ) * 2);
+	if( dun.lev == DUN_LEV_LAST_BOSS )
+		return TRUE;
+
+	rate = LIGHTED_ROOM_MAX_RATE - (randm( labs( dun.lev ) ) * 2);
 
 	if( rate <= 0 )
 		return FALSE;
@@ -1486,7 +1646,7 @@ void	make_door_room( long mx, long my )
 	if( dy < 0 )
 		return;
 
-	make_door( mx, my, mx + dx, my + dy, FACE_MNR_NULL );
+	make_door( mx, my, mx + dx, my + dy, dun.map.obj.mnr[my][mx] );
 }
 
 /***************************************************************
@@ -2026,10 +2186,13 @@ void	make_stairs_randm( char face )
 		x = randm( MAP_MAX_X );
 		y = randm( MAP_MAX_Y );
 
-		if( dun.map.obj.mjr[y][x] == FACE_MJR_FLOOR ){
-			make_stairs( x, y, face );
-			break;
-		}
+		if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
+			continue;
+		if( dun.map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+			continue;
+
+		make_stairs( x, y, face );
+		break;
 	}
 }
 
@@ -2046,8 +2209,13 @@ void	make_stairs( long x, long y, char face )
 
 	if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 		return;
+	if( dun.map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+		return;
 
-	if( (face == FACE_MJR_STAIRS_DOWN)
+	if( face == FACE_MNR_STAIRS_LAST_BOSS ){
+		mjr = FACE_MJR_STAIRS_LAST_BOSS;
+		mnr = FACE_MNR_STAIRS_LAST_BOSS;
+	} else if( (face == FACE_MJR_STAIRS_DOWN)
 			|| (face == FACE_MNR_STAIRS_DOWN) ){
 		if( labs( dun.lev ) >= labs( DUN_MAX_LEV_BASE ) )
 			return;
@@ -2081,6 +2249,8 @@ bool_t	set_statue( long x, long y, long dun_lev, bool_t flg_self )
 
 	if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 		return FALSE;
+	if( dun.map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+		return FALSE;
 	if( !flg_self && (dun.map.chr.mjr[y][x] != FACE_MJR_NULL) )
 		return FALSE;
 
@@ -2166,7 +2336,9 @@ void	set_map_total_bg( long x, long y, long dx, long dy )
 
 	for( yy = by; yy <= ey; yy++ ){
 		for( xx = bx; xx <= ex; xx++ ){
-			set_face_from_map_obj( TRUE, xx, yy, &mjr, &mnr );
+			set_face_from_map_obj(
+					TRUE, xx, yy, &mjr, &mnr,
+					FALSE );
 
 			dun.map.total.mjr[yy][xx] = mjr;
 			dun.map.total.mnr[yy][xx] = mnr;
@@ -2186,7 +2358,9 @@ void	set_map_total_bg( long x, long y, long dx, long dy )
 ***************************************************************/
 
 void	set_face_from_map_obj(
-	bool_t flg_chk_find, long x, long y, char *mjr, char *mnr
+	bool_t flg_chk_find,
+	long x, long y, char *mjr, char *mnr,
+	bool_t flg_gui
 )
 {
 	flg_map_t	flg;
@@ -2204,6 +2378,12 @@ void	set_face_from_map_obj(
 	flg = dun.map.obj.flg[y][x];
 	dep = calc_light_depth( x, y );
 
+	if( (*mjr == FACE_MJR_FLOOR) && (*mnr == FACE_MNR_HOLE) ){
+		if( !flg_gui ){
+			*mjr = FACE_MJR_NULL;
+			*mnr = FACE_MNR_NULL;
+		}
+	}
 	if( chk_flg( flg, FLG_MAP_OBJ_LOOK_WALL ) ){
 		*mjr = FACE_MJR_WALL;
 		*mnr = FACE_MNR_WALL;
@@ -2472,11 +2652,15 @@ void	set_map_total_chr( long x, long y, long dx, long dy )
 							(CURS_ATTR_N_MBR_1
 							+ p->mbr_n);
 				}
+				/* Ʃʤ */
 				if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) ){
-					/* Ʃʤ */
 					flg_rev_rev = TRUE;
 				}
 			} else if( chr_p->kind == CHR_KIND_MNSTR ){
+				/* Ʃʤ */
+				if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) )
+					continue;
+
 				/* 󥹥ä */
 				if( !chk_flg( flg, FLG_MAP_CHR_FIND ) ){
 					/* ĤäƤʤ */
@@ -2890,6 +3074,7 @@ void	find_obj_fill( long x, long y )
 	long	ax, ay;
 	long	sx, sy;
 	long	dx, dy;
+	char	mjr, mnr;
 	sect_t	sect;
 
 	ax = x / AREA_MAX_X * AREA_MAX_X;
@@ -2904,8 +3089,14 @@ void	find_obj_fill( long x, long y )
 			if( dun.map.sect[sy][sx] != sect )
 				continue;
 			if( !chk_flg( dun.map.obj.flg[sy][sx],
-					FLG_MAP_OBJ_PASS ) )
-				continue;
+					FLG_MAP_OBJ_PASS ) ){
+				mjr = dun.map.obj.mjr[sy][sx];
+				mnr = dun.map.obj.mnr[sy][sx];
+				if( mjr != FACE_MJR_FLOOR )
+					continue;
+				if( mnr != FACE_MNR_HOLE )
+					continue;
+			}
 
 			find_obj_fill_field( sx, sy );
 		}
@@ -3158,10 +3349,8 @@ bool_t	chk_find( pos_t *pos1, pos_t *pos2 )
 
 	for( ; x <= x2; x++ ){
 		if( e < 0 ){
-			if( !chk_flg( dun.map.obj.flg[y][x],
-					FLG_MAP_OBJ_PASS ) ){
+			if( !chk_find_pos( x, y ) )
 				return FALSE;
-			}
 		} else {
 			do {
 				y += sgn_y;
@@ -3173,10 +3362,8 @@ bool_t	chk_find( pos_t *pos1, pos_t *pos2 )
 						break;
 				}
 				e -= dx2;
-				if( !chk_flg( dun.map.obj.flg[y][x],
-						FLG_MAP_OBJ_PASS ) ){
+				if( !chk_find_pos( x, y ) )
 					return FALSE;
-				}
 			} while( e >= 0 );
 		}
 		e += dy2;
@@ -3185,6 +3372,26 @@ bool_t	chk_find( pos_t *pos1, pos_t *pos2 )
 	return TRUE;
 }
 
+/***************************************************************
+* ̤뤫ʸ뤫Ĵ٤
+* long x : X ɸ
+* long y : Y ɸ
+* return : ̤뤫?
+***************************************************************/
+
+bool_t	chk_find_pos( long x, long y )
+{
+	if( !clip_pos( x, y ) )
+		return FALSE;
+
+	if( chk_flg( dun.map.obj.flg[y][x], FLG_MAP_OBJ_PASS ) )
+		return TRUE;
+	if( dun.map.obj.mjr[y][x] == FACE_MJR_FLOOR )
+		return TRUE;
+
+	return FALSE;
+}
+
 /***************************************************************
 * µܤΥǡ֤
 * return : µܤΥǡ
@@ -3615,6 +3822,27 @@ bool_t	down_stairs( void )
 	return TRUE;
 }
 
+/***************************************************************
+* 饹ܥؤγʤ򲼤
+* return : 줿?
+***************************************************************/
+
+bool_t	down_stairs_last_boss( void )
+{
+	if( !chk_party_no_mbr( FALSE ) )
+		return FALSE;
+
+	if( !chk_stairs( FACE_MNR_STAIRS_LAST_BOSS ) ){
+		print_msg( FLG_NULL, MSG_ERR_STAIRS_DOWN );
+		return FALSE;
+	}
+
+	print_msg( FLG_NULL, MSG_DOWN_DUN );
+	enter_last_boss_dun();
+
+	return TRUE;
+}
+
 /***************************************************************
 * ʤ겼뤫Ĵ٤
 * long face : ʤμδʸ
@@ -3623,16 +3851,22 @@ bool_t	down_stairs( void )
 
 bool_t	chk_stairs( long face )
 {
+	char	mjr, mnr;
 	party_t	*pty;
 	bool_t	flg;
 	long	i;
 	long	x, y;
 
-	if( (face == FACE_MJR_STAIRS_DOWN)
+	if( face == FACE_MNR_STAIRS_LAST_BOSS ){
+		mjr = FACE_MJR_STAIRS_LAST_BOSS;
+		mnr = FACE_MNR_STAIRS_LAST_BOSS;
+	} else if( (face == FACE_MJR_STAIRS_DOWN)
 			|| (face == FACE_MNR_STAIRS_DOWN) ){
-		face = FACE_MJR_STAIRS_DOWN;
+		mjr = FACE_MJR_STAIRS_DOWN;
+		mnr = FACE_MNR_STAIRS_DOWN;
 	} else {
-		face = FACE_MJR_STAIRS_UP;
+		mjr = FACE_MJR_STAIRS_UP;
+		mnr = FACE_MNR_STAIRS_UP;
 	}
 
 	pty = get_party();
@@ -3650,9 +3884,13 @@ bool_t	chk_stairs( long face )
 
 				mx = pty->mbr[i]->x + x;
 				my = pty->mbr[i]->y + y;
-				if( dun.map.obj.mjr[my][mx] == face ){
-					flg = TRUE;
-				}
+				if( dun.map.obj.mjr[my][mx] != mjr )
+					continue;
+				if( dun.map.obj.mnr[my][mx] != mnr )
+					continue;
+
+				flg = TRUE;
+				break;
 			}
 		}
 		if( !flg )
@@ -3662,6 +3900,24 @@ bool_t	chk_stairs( long face )
 	return TRUE;
 }
 
+/***************************************************************
+* ޥåפ򥹥
+* long x :  x ɸ
+* long y :  y ɸ
+***************************************************************/
+
+void	scroll_map( long x, long y )
+{
+	pos_t	*draw;
+
+	draw = get_map_draw_pos();
+	draw->x = x;
+	draw->y = y;
+
+	chk_scroll( TRUE );
+	redraw_map();
+}
+
 /***************************************************************
 * ʸꤷƥޥåפ򸡺
 * char face_mjr : ᥸㡼δʸ
@@ -3989,4 +4245,3 @@ bool_t	save_dun( void )
 
 	return TRUE;
 }
-
