@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: gf-conf.c,v 1.2 2008/05/10 05:07:27 bowkenken Exp $
+* $Id: gf-conf.c,v 1.12 2008/09/26 10:07:43 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -338,6 +338,9 @@ bool_t	load_game_conf_set_val(
 	} else if( strcmp( id, "multiple_message_line" ) == 0 ){
 		return load_game_conf_mul_line( id, val );
 
+	} else if( strcmp( id, "joystick_number" ) == 0 ){
+		return load_game_conf_joystick_number( id, val );
+
 	} else if( strcmp( id, "vfx_member_cure" ) == 0 ){
 		return load_game_conf_vfx_mbr( FLG_VFX_MBR_CURE, val );
 
@@ -386,8 +389,12 @@ bool_t	load_game_conf_set_val(
 		if( name == NULL )
 			continue;
 
-		if( strncmp( id, name, str_len_std( name ) ) == 0 )
-			return load_game_conf_win_pos( id, val, (win_kind_t)i );
+		if( strncmp( id, name, str_len_std( name ) ) == 0 ){
+			if( load_game_conf_win_pos(
+					id, val, (win_kind_t)i ) ){
+				return TRUE;
+			}
+		}
 	}
 
 	print_lex_err( MSG_S, MSG_ERR_RC_PARSE_DEF_ID );
@@ -415,6 +422,9 @@ bool_t	load_game_conf_set_str(
 	} else if( strcmp( id, "scroll" ) == 0 ){
 		return load_game_conf_scroll( id, str );
 
+	} else if( strcmp( id, "joystick_type" ) == 0 ){
+		return load_game_conf_joystick_kind( id, str );
+
 	} else if( strcmp( id, "color" ) == 0 ){
 		return load_game_conf_color_name( id, str );
 
@@ -598,6 +608,23 @@ bool_t	load_game_conf_scroll( char *id, char *str )
 	return TRUE;
 }
 
+/***************************************************************
+* 祤ƥåμ
+* char *id : ̻
+* char *str : 祤ƥåμ̾
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_joystick_kind( char *id, char *str )
+{
+	joy_kind_t	k;
+
+	k = cv_joystick_name_to_kind( str );
+	call_game_joystick_set_kind( k );
+
+	return TRUE;
+}
+
 /***************************************************************
 * եξ˥Хåե򥯥ꥢ뤫
 * char *id : ̻
@@ -668,6 +695,20 @@ bool_t	load_game_conf_mul_line( char *id, long val )
 	return TRUE;
 }
 
+/***************************************************************
+* 祤ƥåֹ
+* char *id : ̻
+* long val : 祤ƥåֹ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_joystick_number( char *id, long val )
+{
+	call_game_joystick_sel_joy( (joy_kind_t)val );
+
+	return TRUE;
+}
+
 /***************************************************************
 * եΥɥ֤
 * char *id : ̻
@@ -685,25 +726,25 @@ bool_t	load_game_conf_win_pos( char *id, long val, win_kind_t kind )
 	if( name == NULL )
 		return FALSE;
 
-	sprintf( v, "%s_x", name );
+	sn_printf( v, TOKEN_MAX_LEN, "%s_x", name );
 	if( strcmp( id, v ) == 0 ){
 		call_win_pos_set( kind, &val, NULL, NULL, NULL );
 		return TRUE;
 	}
 
-	sprintf( v, "%s_y", name );
+	sn_printf( v, TOKEN_MAX_LEN, "%s_y", name );
 	if( strcmp( id, v ) == 0 ){
 		call_win_pos_set( kind, NULL, &val, NULL, NULL );
 		return TRUE;
 	}
 
-	sprintf( v, "%s_width", name );
+	sn_printf( v, TOKEN_MAX_LEN, "%s_width", name );
 	if( strcmp( id, v ) == 0 ){
 		call_win_pos_set( kind, NULL, NULL, &val, NULL );
 		return TRUE;
 	}
 
-	sprintf( v, "%s_height", name );
+	sn_printf( v, TOKEN_MAX_LEN, "%s_height", name );
 	if( strcmp( id, v ) == 0 ){
 		call_win_pos_set( kind, NULL, NULL, NULL, &val );
 		return TRUE;
@@ -799,6 +840,8 @@ bool_t	load_game_conf_color_name( char *id, char *name )
 		g_crnt_attr_n = CURS_ATTR_N_REV;
 	if( strcmp( name, "err" ) == 0 )
 		g_crnt_attr_n = CURS_ATTR_N_ERR;
+	if( strcmp( name, "menu_normal" ) == 0 )
+		g_crnt_attr_n = CURS_ATTR_N_MENU_NORMAL;
 	if( strcmp( name, "menu_dis" ) == 0 )
 		g_crnt_attr_n = CURS_ATTR_N_MENU_DIS;
 	if( strcmp( name, "menu_sel" ) == 0 )
@@ -1178,6 +1221,9 @@ bool_t	load_game_data_cnf( long n )
 	if( chk_ver_old( &ver, 1, 1, 2 ) )
 		ver_up_game_data_1_1_3();
 
+	if( chk_ver_old( &ver, 1, 2, 11 ) )
+		ver_up_game_data_1_2_12();
+
 	/* եΥ */
 
 	if( fclose( fp ) == EOF )
@@ -1558,6 +1604,7 @@ bool_t	save_game_conf_contents( FILE *fp, const ver_t *ver )
 	flg_err |= !save_game_conf_scroll( fp, ver );
 	flg_err |= !save_game_conf_flg( fp, ver );
 	flg_err |= !save_game_conf_win_pos( fp, ver );
+	flg_err |= !save_game_conf_joystick( fp, ver );
 	flg_err |= !save_game_conf_vfx( fp, ver );
 	flg_err |= !save_game_conf_color( fp, ver );
 	flg_err |= !save_game_conf_key_tab( fp, ver );
@@ -1760,6 +1807,39 @@ bool_t	save_game_conf_win_pos( FILE *fp, const ver_t *ver )
 	return TRUE;
 }
 
+/***************************************************************
+* եΥ祤ƥåΥ
+* FILE *fp : ե
+* const ver_t *ver : եΥС
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	save_game_conf_joystick( FILE *fp, const ver_t *ver )
+{
+	long	n;
+	joy_kind_t	kind;
+	const char	*name;
+
+	if( fp == NULL )
+		return FALSE;
+	if( ver == NULL )
+		return FALSE;
+
+	fprintf( fp, "\n" );
+	fprintf( fp, "# joystick\n" );
+	fprintf( fp, "\n" );
+
+	n = call_game_joystick_get_joy();
+	kind = call_game_joystick_get_kind();
+	name = cv_joystick_kind_to_name( kind );
+
+	fprintf( fp, "joystick_number = %ld\n", (long)n );
+	fprintf( fp, "joystick_type = \"%s\"\n", name );
+	fprintf( fp, "\n" );
+
+	return TRUE;
+}
+
 /***************************************************************
 * եλи̤Υ
 * FILE *fp : ե
@@ -1837,6 +1917,8 @@ bool_t	save_game_conf_color( FILE *fp, const ver_t *ver )
 			CURS_ATTR_N_REV );
 	save_game_conf_color_sub( fp, "err",
 			CURS_ATTR_N_ERR );
+	save_game_conf_color_sub( fp, "menu_normal",
+			CURS_ATTR_N_MENU_NORMAL );
 	save_game_conf_color_sub( fp, "menu_dis",
 			CURS_ATTR_N_MENU_DIS );
 	save_game_conf_color_sub( fp, "menu_sel",
@@ -2005,11 +2087,11 @@ bool_t	save_game_conf_key_tab( FILE *fp, const ver_t *ver )
 
 		if( key_tab[i] != NULL ){
 			if( isprint( (char)i ) ){
-				sprintf( key, "%c", (char)i );
+				sn_printf( key, 15, "%c", (char)i );
 			} else if( ('A' - '@' <= i) && (i <= 'Z' - '@') ){
-				sprintf( key, "^%c", (char)(i + '@') );
+				sn_printf( key, 15, "^%c", (char)(i + '@') );
 			} else {
-				sprintf( key, "\\%04o", (char)i );
+				sn_printf( key, 15, "\\%04o", (char)i );
 			}
 
 			fprintf( fp, "set_key '%s'",
@@ -2119,7 +2201,7 @@ void	back_up_game_conf( void )
 /***************************************************************
 * ɥ֤̾
 * win_kind_t k : ɥμ
-* return : 
+* return : ɥ̾
 ***************************************************************/
 
 const char	*get_win_kind_name( win_kind_t k )
@@ -2153,6 +2235,14 @@ const char	*get_win_kind_name( win_kind_t k )
 		return "window_input_number";
 	case WIN_KIND_INPUT_STRING:
 		return "window_input_string";
+	case WIN_KIND_WORDS:
+		return "window_words";
+	case WIN_KIND_REPLAY:
+		return "window_replay";
+	case WIN_KIND_CLASS_EDIT:
+		return "window_class_edit";
+	case WIN_KIND_CHR_SHEET:
+		return "window_character_sheet";
 	case WIN_KIND_MAX_N:
 		break;
 	}
@@ -2160,6 +2250,51 @@ const char	*get_win_kind_name( win_kind_t k )
 	return NULL;
 }
 
+/***************************************************************
+* 祤ƥåμफ饸祤ƥåμ̾Ѵ
+* joy_kind_t k : 祤ƥåμ
+* return : 祤ƥåμ̾
+***************************************************************/
+
+const char	*cv_joystick_kind_to_name( joy_kind_t k )
+{
+	switch( k ){
+	case JOY_KIND_NULL:
+		break;
+	case JOY_KIND_PS:
+		return "ps like";
+	case JOY_KIND_SATURN:
+		return "saturn like";
+	case JOY_KIND_RAP3:
+		return "real arcade pro.3";
+	case JOY_KIND_MAX_N:
+		break;
+	}
+
+	return "ps-like";
+}
+
+/***************************************************************
+* 祤ƥåμ̾饸祤ƥåμѴ
+* const char *name : 祤ƥåμ̾
+* return : 祤ƥåμ
+***************************************************************/
+
+joy_kind_t	cv_joystick_name_to_kind( const char *name )
+{
+	if( name == NULL )
+		return JOY_KIND_PS;
+
+	if( strcmp( name, "ps like" ) == 0 )
+		return JOY_KIND_PS;
+	if( strcmp( name, "saturn like" ) == 0 )
+		return JOY_KIND_SATURN;
+	if( strcmp( name, "real arcade pro.3" ) == 0 )
+		return JOY_KIND_RAP3;
+
+	return JOY_KIND_PS;
+}
+
 /***************************************************************
 * ֹ椫鿧ʸѴ
 * int color : ֹ
