@@ -102,6 +102,8 @@ anthology_kind_t	trans_idx_to_anthology_kind( long idx );
 long	trans_anthology_kind_to_idx( anthology_kind_t k );
 light_kind_t	trans_idx_to_light_kind( long idx );
 long	trans_light_kind_to_idx( light_kind_t k );
+wish_level_t	trans_idx_to_wish_level( long idx );
+long	trans_wish_level_to_idx( wish_level_t k );
 food_kind_t	trans_idx_to_food_kind( long idx );
 long	trans_food_kind_to_idx( food_kind_t k );
 tobacco_kind_t	trans_idx_to_tobacco_kind( long idx );
@@ -185,11 +187,13 @@ bool_t	load_game_conf_clk_tck( char *id, long val );
 bool_t	load_game_conf_screen_size_auto( char *id, long val );
 bool_t	load_game_conf_screen_size_lines( char *id, long val );
 bool_t	load_game_conf_scroll( char *id, char *str );
+bool_t	load_game_conf_joystick_kind( char *id, char *str );
 bool_t	load_game_conf_clr_key_buf_every( char *id, long val );
 bool_t	load_game_conf_clr_key_buf_menu( char *id, long val );
 bool_t	load_game_conf_space_select( char *id, long val );
 bool_t	load_game_conf_redraw_every_turn( char *id, long val );
 bool_t	load_game_conf_mul_line( char *id, long val );
+bool_t	load_game_conf_joystick_number( char *id, long val );
 bool_t	load_game_conf_win_pos( char *id, long val, win_kind_t kind );
 bool_t	load_game_conf_vfx_mbr( flg_vfx_mbr_t flg, long val );
 bool_t	load_game_conf_vfx_mnstr( flg_vfx_mnstr_t flg, long val );
@@ -235,6 +239,7 @@ bool_t	save_game_conf_screen_size( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_scroll( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_flg( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_win_pos( FILE *fp, const ver_t *ver );
+bool_t	save_game_conf_joystick( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_vfx( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_color( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_color_sub(
@@ -248,6 +253,8 @@ bool_t	save_game_data_cnf_contents(
 );
 void	back_up_game_conf( void );
 const char	*get_win_kind_name( win_kind_t k );
+const char	*cv_joystick_kind_to_name( joy_kind_t k );
+joy_kind_t	cv_joystick_name_to_kind( const char *name );
 char	*cv_color_to_name( int color );
 int	cv_name_to_color( char *name );
 
@@ -354,7 +361,8 @@ bool_t	load_game_data( bool_t flg_txt, long n );
 void	ver_up_game_data_1_0_4( void );
 void	ver_up_game_data_1_1_1( void );
 void	ver_up_game_data_1_1_3( void );
-void	ver_up_game_data_1_1_6( void );
+void	ver_up_game_data_1_2_6( void );
+void	ver_up_game_data_1_2_12( void );
 void	reset_load_game_data( void );
 bool_t	chk_load_game_data( void );
 bool_t	load_game_data_head_ver_1( FILE *fp, ver_t *ver, bool_t flg_txt );
@@ -711,7 +719,7 @@ bool_t	save_game_data_flg_srch_ver_1(
 );
 bool_t	save_game_data_auto_mark_ver_1(
 	FILE *fp, const ver_t *ver, bool_t flg_txt,
-	const auto_mark_t *a
+	const auto_mark_t *a, long auto_mark_n
 );
 bool_t	save_game_data_spell_ls_ver_1(
 	FILE *fp, const ver_t *ver, bool_t flg_txt
