@@ -4,16 +4,25 @@
 extern "C" {
 #endif
 
+void	init_game( void );
+void	reset_static( void );
+void	reset_static_game_main( void );
+void	game_main( void );
+void	game_over( void );
+void	ending( void );
 void	init_arg( void );
 void	chk_arg( int argc, char **argv );
-bool_t	chk_opt_flg( const char *optarg );
+bool_t	chk_opt_num(
+	const char *optarg, long *pn, bool_t flg_err_exit
+);
+bool_t	chk_opt_flg(
+	const char *optarg, bool_t *flg, bool_t flg_err_exit
+);
 void	overwrite_arg();
 void	usage( FILE *fp );
-void	init_game( void );
-void	game_main( void );
 void	exit_game( long code );
 void	close_game( long code );
-void	about_game( void );
+void	about_game( bool_t flg_init );
 void	version( FILE *fp );
 void	child_ps( void );
 void	init_signal( void );
