@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: gmain.c,v 1.104 2008/04/18 18:33:40 bowkenken Exp $
+* $Id: gmain.c,v 1.116 2008/10/07 02:59:19 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -62,9 +62,9 @@
 
 static long	flg_term;
 
-#if	HAVE_TERMIOS_H
+#if	defined( HAVE_TERMIOS_H )
 static struct termios	pre_tbuf;
-#elif	HAVE_TERMIO_H
+#elif	defined( HAVE_TERMIO_H )
 static struct termio	pre_tbuf;
 #endif
 
@@ -75,7 +75,7 @@ static struct termio	pre_tbuf;
 #define	OPT_ARG_MAX_LEN	128
 #define	STR_OPT	"GCUT:D:S:l:k:e:s:p:md:Vvh"
 
-#if	HAVE_GETOPT_LONG
+#ifdef	HAVE_GETOPT_LONG
 static struct option	long_opt[] = {
 	{ "gui",           no_argument,       NULL, 'G' },
 	{ "cui",           no_argument,       NULL, 'C' },
@@ -106,16 +106,20 @@ char	str_usage[] = {
 	"  -G, --gui                set GUI mode\n"
 	"  -C, --cui                set CUI mode\n"
 	"  -U, --gui-and-cui        set GUI and CUI mode\n"
+#if	defined( DEBUG ) || !defined( NDEBUG )
 	"  -T, --text-mode=POINT    set TEXT mode:\n"
 	"                             point of font, ON or OFF\n"
 	"  -D, --sel-graph-dir=DIR  select graphic directory:\n"
 	"                             graphic directory\n"
 	"  -S, --map-scale=N        set scale of map character:\n"
 	"                             scale(%%)\n"
+#endif
 	"  -l, --language=LANGUAGE  set language:\n"
 	"                             language\n"
+#if	defined( DEBUG ) || !defined( NDEBUG )
 	"  -k, --clear-key-buf=FLAG clear key buffer:\n"
 	"                             ON or OFF\n"
+#endif
 	"  -e, --echo-key=FLAG      echo key:\n"
 	"                             ON or OFF\n"
 	"  -s, --use-curses=FLAG    use curses library:\n"
@@ -184,6 +188,7 @@ void	init_game( void )
 	init_misc_menu();
 	init_main_menu();
 	init_request();
+	init_last_boss();
 
 	init_mbr_formation();
 
@@ -240,6 +245,7 @@ void	reset_static( void )
 	reset_static_town();
 	reset_static_trap();
 	reset_static_turn();
+	reset_static_last_boss();
 
 #if	defined( D_GTK )
 #elif	defined( D_WS )
@@ -267,8 +273,11 @@ void	game_main( void )
 {
 	chk_scroll( TRUE );
 	redraw_all();
+	scroll_center_appear_party();
 
-	if( !g_flg_debug ){
+	if( g_flg_debug ){
+		menu_load_wiz();
+	} else {
 		print_msg( FLG_NULL, MSG_HINT_CANCEL_MENU_1 );
 		print_msg( FLG_NULL, MSG_HINT_CANCEL_MENU_2 );
 
@@ -282,6 +291,39 @@ void	game_main( void )
 	}
 }
 
+/***************************************************************
+* ࡦС
+***************************************************************/
+
+void	game_over( void )
+{
+	change_scene( SCENE_N_GAME_OVER );
+
+	print_msg( FLG_MSG_NO_MORE_PREV, MSG_S, MSG_GAME_OVER );
+	while( 1 ){
+		switch( get_key( FALSE ) ){
+		case CASE_EXIT:
+			exit_game( EXIT_SUCCESS );
+		}
+	}
+}
+
+/***************************************************************
+* ǥ
+***************************************************************/
+
+void	ending( void )
+{
+	change_scene( SCENE_N_ENDING );
+
+	while( 1 ){
+		switch( get_key( FALSE ) ){
+		case CASE_EXIT:
+			exit_game( EXIT_SUCCESS );
+		}
+	}
+}
+
 /***************************************************************
 * ν
 ***************************************************************/
@@ -343,13 +385,13 @@ void	chk_arg( int argc, char **argv )
 
 	while( 1 ){
 		long	c;
-#if	HAVE_GETOPT_LONG
+#ifdef	HAVE_GETOPT_LONG
 		int	opt_idx;
 #endif	/* HAVE_GETOPT_LONG */
 
-#if	HAVE_GETOPT_LONG
+#if	defined( HAVE_GETOPT_LONG )
 		c = getopt_long( argc, argv, STR_OPT, long_opt, &opt_idx );
-#elif	HAVE_GETOPT
+#elif	defined( HAVE_GETOPT )
 		c = getopt( argc, argv, STR_OPT );
 #else	/* HAVE_GETOPT_LONG */
 		n++;
@@ -389,7 +431,7 @@ void	chk_arg( int argc, char **argv )
 			}
 			break;
 		case 'D':
-			set_str_dir_sel_graph( optarg );
+			call_set_str_dir_sel_graph( optarg );
 			break;
 		case 'S':
 			chk_opt_num( optarg, &g_graph_scale_rate, TRUE );
@@ -594,7 +636,7 @@ void	close_game( long code )
 {
 	close_item();
 
-	curs_move( WIN_MSG_DRAW_MIN_Y, SCREEN_MIN_X );
+	curs_move( get_max_line() - 1, SCREEN_MIN_X );
 	reset_scrn();
 
 	fflush( stdout );
@@ -613,10 +655,28 @@ void	close_game( long code )
 * ɽ
 ***************************************************************/
 
-void	about_game( void )
+void	about_game( bool_t flg_init )
 {
 	static char	tmp[1023 + 1];
 
+#if	defined( D_WS ) || defined( D_GTK ) || defined( D_DOWS )
+	if( flg_init )
+		return;
+#endif
+
+	if( g_flg_gui ){
+		const long	max_len = 80 * 40;
+		static char	str[80 * 40 + 1];
+
+		sn_printf( str, max_len, MSG_FMT_COPYRIGHT,
+				LS_STR_COPYRIGHT );
+		str_max_n_cat( str, MSG_ABOUT_GAME, max_len );
+
+		call_gui_words_draw_width( str, 80 );
+
+		return;
+	}
+
 	curs_move( WIN_MSG_DRAW_MIN_Y, SCREEN_MIN_X );
 
 	dis_scrn();
@@ -911,16 +971,16 @@ void	ena_term_unix( void )
 #if	(HAVE_TERMIOS_H || HAVE_TERMIO_H)
 	if( flg_term == 1 ){
 		/* ¤Τ */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 		struct termios	tbuf;
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 		struct termio	tbuf;
 # endif
 
 		/* Υå */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 		tcgetattr( fileno( stdin ), &tbuf );
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 		ioctl( 0, TCGETA, &tbuf );
 # endif
 
@@ -942,12 +1002,12 @@ void	ena_term_unix( void )
 		}
 
 		/* Υå */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 		if( tcsetattr( fileno( stdin ), TCSANOW, &tbuf ) != 0 ){
 			fprintf( stderr, MSG_S, MSG_ERR_SET_TTY );
 			exit_game( EXIT_FAILURE );
 		}
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 		if( ioctl( 0, TCSETAF, &tbuf ) != 0 ){
 			fprintf( stderr, MSG_S, MSG_ERR_SET_TTY );
 			exit_game( EXIT_FAILURE );
@@ -967,9 +1027,9 @@ void	dis_term_unix( void )
 		return;
 
 	if( flg_term == 1 ){
-#if	HAVE_TERMIOS_H
+#if	defined( HAVE_TERMIOS_H )
 		tcsetattr( fileno( stdin ), TCSANOW, &pre_tbuf );
-#elif	HAVE_TERMIO_H
+#elif	defined( HAVE_TERMIO_H )
 		ioctl( 0, TCSETAF, &pre_tbuf );
 #endif
 	}
@@ -1004,9 +1064,9 @@ void	set_key_wait_vtime( long wait_vtime )
 {
 #if	(HAVE_TERMIOS_H || HAVE_TERMIO_H)
 	/* ¤Τ */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 	struct termios	tbuf;
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 	struct termio	tbuf;
 # endif
 
@@ -1014,9 +1074,9 @@ void	set_key_wait_vtime( long wait_vtime )
 		return;
 
 	/* Υå */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 	tcgetattr( fileno( stdin ), &tbuf );
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 	ioctl( 0, TCGETA, &tbuf );
 # endif
 
@@ -1024,12 +1084,12 @@ void	set_key_wait_vtime( long wait_vtime )
 	tbuf.c_cc[VTIME] = wait_vtime;
 
 	/* Υå */
-# if	HAVE_TERMIOS_H
+# if	defined( HAVE_TERMIOS_H )
 	if( tcsetattr( fileno( stdin ), TCSANOW, &tbuf ) != 0 ){
 		fprintf( stderr, MSG_S, MSG_ERR_SET_TTY );
 		exit_game( EXIT_FAILURE );
 	}
-# elif	HAVE_TERMIO_H
+# elif	defined( HAVE_TERMIO_H )
 	if( ioctl( 0, TCSETAF, &tbuf ) != 0 ){
 		fprintf( stderr, MSG_S, MSG_ERR_SET_TTY );
 		exit_game( EXIT_FAILURE );
