@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mmenu-tab.h,v 1.26 2008/04/03 08:17:01 bowkenken Exp $
+* $Id: mmenu-tab.h,v 1.38 2008/10/05 10:59:55 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	MMENU_TAB_H
@@ -34,6 +34,7 @@
 /* ᥤ󡦥˥塼 */
 
 #include	"request.h"
+#include	"joy-kind.h"
 
 #include	"request-prot.h"
 
@@ -316,7 +317,7 @@ menu_t	ls_menu_main[] = {
 	{
 		"/auto mark",
 		"<Branch>",
-		'A',
+		'Z',
 		"/ z",
 		N_MSG_MENU_SEL_OBJ_AUTO, "",
 		cb_menu_auto_mark, NULL,
@@ -395,6 +396,60 @@ menu_t	ls_menu_main[] = {
 		cb_menu_misc, NULL,
 		FLG_NULL,
 	},
+	{
+		"/misc/info",
+		"<Branch>",
+		'I',
+		"",
+		N_MSG_MENU_INFO, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/info/hint",
+		"<Item>",
+		'H',
+		"?",
+		N_MSG_MENU_MAP_HINT, "",
+		cb_menu_show_map_hint, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/info/char data",
+		"<OpenWinItem>",
+		'C',
+		"C",
+		N_MSG_MENU_CHR_DATA, "",
+		cb_menu_chr_data, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/info/list message",
+		"<OpenWinItem>",
+		'L',
+		"g",
+		N_MSG_MENU_LS_MSG, "",
+		cb_menu_ls_msg, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/info/list monster",
+		"<Item>",
+		'M',
+		"",
+		N_MSG_MENU_LS_MNSTR, "",
+		cb_menu_ls_mnstr, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/info/list request",
+		"<Branch>",
+		'R',
+		"r",
+		N_MSG_MENU_LS_REQUEST, "",
+		cb_menu_request_mbr, NULL,
+		FLG_NULL,
+	},
 	{
 		"/misc/continue exec",
 		"<Item>",
@@ -431,6 +486,15 @@ menu_t	ls_menu_main[] = {
 		cb_menu_stairs_down, NULL,
 		FLG_NULL,
 	},
+	{
+		"/misc/stairs/last boss",
+		"<Item>",
+		'D',
+		"",
+		N_MSG_MENU_STAIRS_LAST_BOSS, "",
+		cb_menu_stairs_last_boss, NULL,
+		FLG_NULL,
+	},
 	{
 		"/misc/search",
 		"<Branch>",
@@ -440,24 +504,6 @@ menu_t	ls_menu_main[] = {
 		cb_menu_srch, NULL,
 		FLG_NULL,
 	},
-	{
-		"/misc/hint",
-		"<Item>",
-		'H',
-		"?",
-		N_MSG_MENU_MAP_HINT, "",
-		cb_menu_show_map_hint, NULL,
-		FLG_NULL,
-	},
-	{
-		"/misc/char data",
-		"<OpenWinItem>",
-		'D',
-		"C",
-		N_MSG_MENU_CHR_DATA, "",
-		cb_menu_chr_data, NULL,
-		FLG_NULL,
-	},
 	{
 		"/misc/gold",
 		"<Branch>",
@@ -512,33 +558,6 @@ menu_t	ls_menu_main[] = {
 		cb_menu_crsr, NULL,
 		FLG_NULL,
 	},
-	{
-		"/misc/list message",
-		"<OpenWinItem>",
-		'M',
-		"g",
-		N_MSG_MENU_LS_MSG, "",
-		cb_menu_ls_msg, NULL,
-		FLG_NULL,
-	},
-	{
-		"/misc/list monster",
-		"<Item>",
-		'L',
-		"",
-		N_MSG_MENU_LS_MNSTR, "",
-		cb_menu_ls_mnstr, NULL,
-		FLG_NULL,
-	},
-	{
-		"/misc/list request",
-		"<Branch>",
-		'R',
-		"r",
-		N_MSG_MENU_LS_REQUEST, "",
-		cb_menu_request_mbr, NULL,
-		FLG_NULL,
-	},
 /* config begin */
 	{
 		"/misc/config",
@@ -1021,6 +1040,69 @@ menu_t	ls_menu_main[] = {
 		cb_menu_func, NULL,
 		FLG_NULL,
 	},
+	{
+		"/misc/config/gui",
+		"<Branch>",
+		'G',
+		"",
+		N_MSG_MENU_GUI, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick",
+		"<Branch>",
+		'J',
+		"",
+		N_MSG_MENU_JOYSTICK, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick/select joystick",
+		"<Branch>",
+		'D',
+		"",
+		N_MSG_MENU_JOYSTICK_SEL_N, "",
+		cb_menu_joystick_sel_n_init, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick/select kind",
+		"<Branch>",
+		'T',
+		"",
+		N_MSG_MENU_JOYSTICK_SEL_KIND, "",
+		cb_menu_joystick_sel_kind_init, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick/select kind/ps",
+		"<RadioItem>",
+		'P',
+		"",
+		N_MSG_MENU_JOYSTICK_PS, "",
+		cb_menu_joystick_sel_kind, (void *)JOY_KIND_PS,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick/select kind/saturn",
+		"<RadioItem>",
+		'S',
+		"",
+		N_MSG_MENU_JOYSTICK_SATURN, "",
+		cb_menu_joystick_sel_kind, (void *)JOY_KIND_SATURN,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/joystick/select kind/rap3",
+		"<RadioItem>",
+		'R',
+		"",
+		N_MSG_MENU_JOYSTICK_RAP3, "",
+		cb_menu_joystick_sel_kind, (void *)JOY_KIND_RAP3,
+		FLG_NULL,
+	},
 	{
 		"/misc/config/user menu select",
 		"<Branch>",
@@ -1136,20 +1218,146 @@ menu_t	ls_menu_main[] = {
 	},
 	{
 		"/misc/system/wizard/add turn",
-		"<OpenWinItem>",
+		"<Branch>",
 		'T',
 		"",
 		N_MSG_MENU_WIZARD_ADD_TURN, "",
-		cb_menu_wizard_add_turn, NULL,
+		NULL, NULL,
 		FLG_NULL,
 	},
 	{
-		"/misc/system/wizard/level up",
+		"/misc/system/wizard/add turn/title",
+		"<Title>",
+		'\0',
+		"",
+		N_MSG_TTL_MENU_WIZARD_ADD_TURN, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/add turn/hour",
 		"<OpenWinItem>",
+		'H',
+		"",
+		N_MSG_MENU_WIZARD_ADD_TURN_HOUR, "",
+		cb_menu_wizard_add_turn_hour, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/add turn/minute",
+		"<OpenWinItem>",
+		'M',
+		"",
+		N_MSG_MENU_WIZARD_ADD_TURN_MINUTE, "",
+		cb_menu_wizard_add_turn_minute, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/add turn/turn",
+		"<OpenWinItem>",
+		'T',
+		"",
+		N_MSG_MENU_WIZARD_ADD_TURN_TURN, "",
+		cb_menu_wizard_add_turn_turn, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/up exp",
+		"<OpenWinItem>",
+		'X',
+		"",
+		N_MSG_MENU_WIZARD_UP_EXP, "",
+		cb_menu_wizard_up_exp, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/up level",
+		"<Branch>",
 		'V',
 		"",
-		N_MSG_MENU_WIZARD_LEV_UP, "",
-		cb_menu_wizard_lev_up, NULL,
+		N_MSG_MENU_WIZARD_UP_LEV, "",
+		cb_menu_wizard_up_lev, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene",
+		"<Branch>",
+		'C',
+		"",
+		N_MSG_MENU_WIZARD_SCENE, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss",
+		"<Branch>",
+		'L',
+		"",
+		N_MSG_MENU_WIZARD_SCENE_LAST_BOSS, "",
+		NULL, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss/enter",
+		"<Item>",
+		'A',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_ENTER, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_ENTER,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss/encounter",
+		"<Item>",
+		'B',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_ENCOUNTER, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_ENCOUNTER,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss/exel",
+		"<Item>",
+		'C',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_EXEL, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_EXEL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss/exeler",
+		"<Item>",
+		'D',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_EXELER, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_EXELER,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/last boss/xex",
+		"<Item>",
+		'E',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_XEX, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_XEX,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/game over",
+		"<Item>",
+		'O',
+		"",
+		N_MSG_MENU_WIZARD_SCENE_GAME_OVER, "",
+		cb_menu_wizard_scene, (void *)SCENE_N_GAME_OVER,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/scene/ending",
+		"<Item>",
+		'E',
+		"",
+		N_MSG_MENU_WIZARD_SCENE_ENDING, "",
+		cb_menu_wizard_scene, (void *)SCENE_N_ENDING,
 		FLG_NULL,
 	},
 	{
@@ -1323,6 +1531,7 @@ menu_t	ls_menu_sel_obj[] = {
 };
 
 menu_t	ls_menu_mbr[1 + MBR_MAX_N + PET_MAX_N + 1];
+menu_t	ls_menu_abl[1 + ABL_KIND_MAX_N * 2 + 1];
 
 menu_t	ls_menu_dist[] = {
 	{
@@ -1756,6 +1965,15 @@ menu_t	ls_menu_item_sel[] = {
 		cb_menu_item_fill_lamp, (void *)ACT_KIND_POTION_FILL,
 		FLG_NULL,
 	},
+	{
+		"/item sel/rub",
+		"<Item>",
+		'F',
+		"",
+		N_MSG_MENU_ITEM_RUB, "",
+		cb_menu_item_use, (void *)ACT_KIND_LAMP_RUB,
+		FLG_NULL,
+	},
 	{
 		"/item sel/equip",
 		"<Item>",
