@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mmenu.c,v 1.42 2008/08/10 23:42:52 bowkenken Exp $
+* $Id: mmenu.c,v 1.43 2008/10/07 02:59:19 bowkenken Exp $
 ***************************************************************/
 
 #define	MMENU_C
@@ -2565,8 +2565,7 @@ char	*cb_menu_fight( menu_t **pp, menu_t **dflt )
 
 char	*cb_menu_load_wiz( menu_t **pp, menu_t **dflt )
 {
-	if( menu_load( TRUE ) )
-		reset_all_save( TRUE );
+	menu_load_wiz();
 
 	return STR_MENU_CANCEL;
 }
@@ -2575,14 +2574,29 @@ char	*cb_menu_load_wiz( menu_t **pp, menu_t **dflt )
 
 char	*cb_menu_save_wiz( menu_t **pp, menu_t **dflt )
 {
-	if( menu_save( TRUE ) )
-		reset_all_save( TRUE );
+	menu_save_wiz();
 
 	return STR_MENU_CANCEL;
 }
 
 /**/
 
+void	menu_load_wiz( void )
+{
+	if( menu_load( TRUE ) )
+		reset_all_save( TRUE );
+}
+
+/**/
+
+void	menu_save_wiz( void )
+{
+	if( menu_save( TRUE ) )
+		reset_all_save( TRUE );
+}
+
+/**/
+
 bool_t	menu_load( bool_t flg_txt )
 {
 	long	n;
