@@ -31,6 +31,9 @@ void	set_act_mnstr_mimic( mnstr_t *p );
 void	set_act_mnstr_vampire( mnstr_t *p );
 void	set_act_mnstr_incubus( mnstr_t *p );
 void	set_act_mnstr_succubus( mnstr_t *p );
+void	set_act_mnstr_exel( mnstr_t *p );
+void	set_act_mnstr_exeler( mnstr_t *p );
+void	set_act_mnstr_xex( mnstr_t *p );
 
 #ifdef __cplusplus
 }
