@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mnstr-act.c,v 1.15 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: mnstr-act.c,v 1.20 2008/10/07 00:41:05 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -59,6 +59,19 @@
 /* ν̱ɥФΨΣܡ */
 #define	TOWNER_EXIT_DOOR_RATE_NIGHT	((rate_t)1)
 
+/***************************************************************/
+
+static long	static_set_act_mnstr_giant_rat__increase_turn;
+
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_mnstr_act( void )
+{
+	static_set_act_mnstr_giant_rat__increase_turn = -1;
+}
+
 /***************************************************************
 * ƥ󥹥ζ̤Υ
 ***************************************************************/
@@ -291,6 +304,9 @@ void	set_act_towner_exit_door( mnstr_t *p )
 
 void	set_act_mnstr_friendly( mnstr_t *p )
 {
+	if( p == NULL )
+		return;
+
 	if( set_act_tame_mnstr( p ) )
 		return;
 
@@ -584,7 +600,9 @@ void	set_act_mnstr_spore( mnstr_t *p )
 
 void	set_act_mnstr_giant_rat( mnstr_t *p )
 {
-	static long	increase_turn = -1;
+	long	increase_turn = -1;
+
+	increase_turn = static_set_act_mnstr_giant_rat__increase_turn;
 
 	if( p == NULL )
 		return;
@@ -824,3 +842,54 @@ void	set_act_mnstr_succubus( mnstr_t *p )
 /*  */
 /*  */
 
+/***************************************************************
+* Υ
+* mnstr_t *p : 󥹥
+***************************************************************/
+
+void	set_act_mnstr_exel( mnstr_t *p )
+{
+	set_act_mnstr_exeler( p );
+}
+
+/***************************************************************
+* 顼Υ
+* mnstr_t *p : 󥹥
+***************************************************************/
+
+void	set_act_mnstr_exeler( mnstr_t *p )
+{
+	long	x, y;
+
+	if( p == NULL )
+		return;
+
+	x = p->x;
+	y = p->y;
+	clr_map_chr( p );
+
+	p->work.hyper_boss++;
+	p->work.hyper_boss %= (HYPER_BOSS_PERFECT + 1);
+	if( p->work.hyper_boss <= HYPER_BOSS_NULL )
+		p->work.hyper_boss = HYPER_BOSS_NULL + 1;
+
+	p->work.flg_hyper_boss = !p->work.flg_hyper_boss;
+
+	p->x = x;
+	p->y = y;
+	set_map_chr( p );
+
+	/**/
+//@@@	set_act_mnstr_std( p );
+}
+
+/***************************************************************
+* Υ
+* mnstr_t *p : 󥹥
+***************************************************************/
+
+void	set_act_mnstr_xex( mnstr_t *p )
+{
+//@@@	set_act_mnstr_std( p );
+}
+
