@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mnstr.c,v 1.195 2008/05/09 12:58:31 bowkenken Exp $
+* $Id: mnstr.c,v 1.203 2008/10/07 08:29:52 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -68,10 +68,14 @@ static mnstr_t	mnstr_used;
 /* ̤ */
 static mnstr_t	mnstr_free;
 
+/* 饹ܥ */
+static mnstr_t	*g_last_boss[LAST_BOSS_KIND_MAX_N];
+
 /***************************************************************
 * 󥹥Υǡơ֥
 ***************************************************************/
 
+#define	FLG_DIVIDE_MNSTR_LEV_TAB_STAGE	1
 #include	"mnstr-tab.h"
 
 /***************************************************************
@@ -80,7 +84,8 @@ static mnstr_t	mnstr_free;
 
 void	init_mnstr( void )
 {
-	long	min_lev, max_lev;
+	bool_t	ls_flg_chk_mnstr_kind[MNSTR_KIND_MAX_N];
+	char	*name;
 	long	i, max_i;
 
 	max_i = sizeof( mnstr_tab ) / sizeof( mnstr_tab[0] );
@@ -91,26 +96,147 @@ void	init_mnstr( void )
 		mnstr_tab[i].name = MSG( mnstr_tab[i].name_n );
 	}
 
-	min_lev = 0;
-	max_i = sizeof( mnstr_lev_tab ) / sizeof( mnstr_lev_tab[0] );
-	for( i = 0; i < max_i; i++ ){
-		mnstr_lev_tab_t	*tab;
+#if	FLG_DIVIDE_MNSTR_LEV_TAB_STAGE
+	{
+		long	n;
 
-		tab = &(mnstr_lev_tab[i]);
+		n = 0;
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_town, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_1, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_2, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_3, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_4, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_5, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_6, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_7, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_8, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_9, &n );
+		init_mnstr_lev_tab( mnstr_lev_tab_stage_10, &n );
+	}
+#else
+	{
+		long	min_lev, max_lev;
 
-		if( tab->kind == MNSTR_KIND_NULL )
+		min_lev = 0;
+		max_i = sizeof( mnstr_lev_tab )
+				/ sizeof( mnstr_lev_tab[0] );
+		for( i = 0; i < max_i; i++ ){
+			mnstr_lev_tab_t	*tab;
+
+			tab = &(mnstr_lev_tab[i]);
+
+			if( tab->kind == MNSTR_KIND_NULL )
+				break;
+
+			min_lev += tab->add_lev;
+			max_lev = min_lev + tab->range_lev - 1;
+
+			tab->min_lev = min_lev;
+			tab->max_lev = max_lev;
+		}
+	}
+#endif
+
+	// åѥơ֥
+
+	for( i = 0; i < MNSTR_KIND_MAX_N; i++ )
+		ls_flg_chk_mnstr_kind[i] = FALSE;
+
+	// ܥå
+
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		if( mnstr_boss_lev_tab[i].kind == MNSTR_KIND_NULL )
 			break;
 
-		min_lev += tab->add_lev;
-		max_lev = min_lev + tab->range_lev - 1;
-
-		tab->min_lev = min_lev;
-		tab->max_lev = max_lev;
+		ls_flg_chk_mnstr_kind[mnstr_boss_lev_tab[i].kind] = TRUE;
 	}
 
+	// 饹ܥå
+
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		if( mnstr_last_boss_lev_tab[i].kind == MNSTR_KIND_NULL )
+			break;
+
+		ls_flg_chk_mnstr_kind[mnstr_last_boss_lev_tab[i].kind]
+				= TRUE;
+	}
+
+	// ʣå
+
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		if( mnstr_lev_tab[i].kind == MNSTR_KIND_NULL )
+			break;
+
+		if( ls_flg_chk_mnstr_kind[mnstr_lev_tab[i].kind] ){
+			name = get_mnstr_name( mnstr_lev_tab[i].kind );
+			print_msg( FLG_MSG_DEBUG,
+					MSG_ERR_MNSTR_LEV_TAB_DUP,
+					name );
+		}
+		ls_flg_chk_mnstr_kind[mnstr_lev_tab[i].kind] = TRUE;
+	}
+
+	// ̤å
+
+	for( i = MNSTR_KIND_NULL + 1; i < MNSTR_KIND_MAX_N; i++ ){
+		if( !ls_flg_chk_mnstr_kind[i] ){
+			name = get_mnstr_name( i );
+			print_msg( FLG_MSG_DEBUG,
+					MSG_ERR_MNSTR_LEV_TAB_NOT_DEF,
+					name );
+		}
+	}
+
+	// ơ֥ͤͤջ
+
+	{
+		long	pre_lev;
+
+		pre_lev = 0;
+		for( i = 0; i < LOOP_MAX_1000; i++ ){
+			if( mnstr_lev_tab[i].kind == MNSTR_KIND_NULL )
+				break;
+
+			mnstr_lev_tab[i].add_lev = mnstr_lev_tab[i].min_lev
+					- pre_lev;
+			mnstr_lev_tab[i].range_lev = mnstr_lev_tab[i].max_lev
+					- mnstr_lev_tab[i].min_lev + 1;
+
+			pre_lev = mnstr_lev_tab[i].min_lev;
+		}
+	}
+
+	for( i = 0; i < LAST_BOSS_KIND_MAX_N; i++ )
+		g_last_boss[i] = NULL;
+
 	reset_mnstr();
 }
 
+/***************************************************************
+* 󥹥Υ٥롦ơ֥
+***************************************************************/
+
+void	init_mnstr_lev_tab( mnstr_lev_tab_t stage[], long *n )
+{
+	long	i;
+
+	if( stage == NULL )
+		return;
+	if( n == NULL )
+		return;
+
+	for( i = 0; i < MNSTR_KIND_MAX_N; i++ ){
+		if( stage[i].kind == MNSTR_KIND_NULL )
+			break;
+		if( *n >= MNSTR_KIND_MAX_N )
+			break;
+
+		mnstr_lev_tab[*n] = stage[i];
+		(*n)++;
+	}
+	mnstr_lev_tab[*n].kind = MNSTR_KIND_NULL;
+}
+
 /***************************************************************
 * ƥåѿꥻå
 ***************************************************************/
@@ -279,6 +405,8 @@ void	new_mnstr( mnstr_t *mnstr )
 
 	mnstr->fx_data.bless = 0;
 
+	mnstr->contribution = 0;
+
 	mnstr->mnstr_kind = MNSTR_KIND_NULL;
 	mnstr->mnstr_kind_org = MNSTR_KIND_NULL;
 	mnstr->mnstr_tab = NULL;
@@ -287,15 +415,23 @@ void	new_mnstr( mnstr_t *mnstr )
 	mnstr->flg_pet = FALSE;
 
 	mnstr->work.flg = FLG_NULL;
+	mnstr->work.hyper_boss = HYPER_BOSS_NULL;
+	mnstr->work.flg_hyper_boss = FALSE;
 	mnstr->work.caught = 0;
 	mnstr->work.catch_chr = NULL;
 	mnstr->work.link_chr = NULL;
+	mnstr->work.teleport_n = 0;
+	mnstr->work.teleport_party_n = 0;
+	mnstr->work.flg_can_wish = TRUE;
 	mnstr->work.request = NULL;
 	mnstr->work.towner_dx = 0;
 	mnstr->work.towner_dy = 0;
 	mnstr->work.enter_door = NULL;
 	mnstr->work.dismissal_turn = 0;
 	mnstr->work.flg_used_energy_drain = FALSE;
+	mnstr->work.sync_vfx_num = 0;
+	mnstr->work.sync_vfx_x = MAP_DEL_X;
+	mnstr->work.sync_vfx_y = MAP_DEL_Y;
 
 	mnstr->graph_name[0] = '\0';
 }
@@ -592,6 +728,15 @@ bool_t	make_mnstr_special( mnstr_t *p, long dun_lev )
 			break;
 		}
 
+		break;
+	case MNSTR_KIND_EXEL:
+		g_last_boss[LAST_BOSS_KIND_EXEL] = p;
+		break;
+	case MNSTR_KIND_EXELER:
+		g_last_boss[LAST_BOSS_KIND_EXELER] = p;
+		break;
+	case MNSTR_KIND_XEX:
+		g_last_boss[LAST_BOSS_KIND_XEX] = p;
 		break;
 	}
 
@@ -2783,6 +2928,17 @@ mnstr_kind_t	get_mnstr_kind_from_face( char mjr, char mnr )
 	return k;
 }
 
+/***************************************************************
+* 饹ܥ֤
+* last_boss_kind_t kind : 饹ܥμ
+* return : 饹ܥ
+***************************************************************/
+
+mnstr_t	*get_last_boss( last_boss_kind_t kind )
+{
+	return g_last_boss[kind];
+}
+
 /***************************************************************
 * 󥹥γƺѥ꡼ꥹȤ֤
 * return : 󥹥Υ꡼ꥹ
