@@ -24,6 +24,8 @@ void	appear_party( void );
 bool_t	appear_party_randm( void );
 bool_t	appear_party_srch( void );
 void	appear_party_force( void );
+void	appear_party_last_boss( void );
+void	appear_party_map_event( char mnr );
 long	get_party_dun_max_lev( void );
 void	scroll_center_appear_party( void );
 void	clr_map_all_mbr( void );
