@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: party.c,v 1.200 2008/03/25 22:27:41 bowkenken Exp $
+* $Id: party.c,v 1.220 2008/10/07 10:27:00 bowkenken Exp $
 ***************************************************************/
 
 #define	PARTY_C
@@ -33,6 +33,10 @@
 
 /**/
 
+#define	FLG_NICOTINE_TEST	0
+
+/**/
+
 #define	DFLT_EXP	500
 #define	DFLT_GOLD	3000
 
@@ -177,7 +181,10 @@ void	init_party( void )
 
 	party.square.x = 1;
 	party.square.y = 1;
+	party.square.pre_x = 1;
+	party.square.pre_y = 1;
 	party.square.dir = SQUARE_DIR_NORTH;
+	party.square.pre_dir = SQUARE_DIR_NORTH;
 	party.square.chr_pos_n[0][0] = 0;
 	party.square.chr_pos_n[0][1] = MBR_NO_POS;
 	party.square.chr_pos_n[0][2] = 1;
@@ -198,6 +205,14 @@ void	init_party( void )
 		init_party_dflt();
 }
 
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_party( void )
+{
+}
+
 /**/
 
 void	init_party_dflt( void )
@@ -275,6 +290,7 @@ void	init_party_dflt( void )
 
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		sub_kind.light = LIGHT_KIND_TORCH;
+		sub_kind.wish_level = WISH_LEVEL_NULL;
 		init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
 				TRUE, FALSE, TRUE );
 	}
@@ -490,7 +506,7 @@ void	init_party_debug( void )
 
 	for( i = 0; i < LS_MBR_MAX_N; i++ ){
 		ls_mbr[i]->dun_max_lev_floor = 0;
-		ls_mbr[i]->dun_max_lev_base = 128;
+		ls_mbr[i]->dun_max_lev_base = DUN_MAX_LEV_BASE;
 
 		ls_mbr[i]->fx_data.friend_turn = 0;
 	}
@@ -503,6 +519,7 @@ void	init_party_debug( void )
 				TRUE, FALSE, TRUE );
 
 		sub_kind.light = LIGHT_KIND_LANTERN;
+		sub_kind.wish_level = WISH_LEVEL_NULL;
 		init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
 				TRUE, FALSE, TRUE );
 
@@ -888,14 +905,31 @@ void	init_party_debug( void )
 	init_mbr_item( ls_mbr[i], ITEM_KIND_CHEST, NULL,
 			TRUE, FALSE, FALSE );
 
+	i = 4;
+	sub_kind.light = LIGHT_KIND_LAMP;
+	sub_kind.wish_level = WISH_LEVEL_ARMAGEDDON;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 4;
+	sub_kind.light = LIGHT_KIND_LAMP;
+	sub_kind.wish_level = WISH_LEVEL_DAEMON;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 4;
+	sub_kind.light = LIGHT_KIND_LAMP;
+	sub_kind.wish_level = WISH_LEVEL_DJINNI;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
+			FALSE, FALSE, FALSE );
+
 	/**/
 
-#if	0
-/*@@@*/
-	i = 0;
-	ls_mbr[i]->fx_data.nicotine.poisoning_rate = _100_PERCENT;
-	ls_mbr[i]->fx_data.nicotine.lost_rate = _100_PERCENT;
-#endif
+	if( FLG_NICOTINE_TEST ){
+		i = 0;
+		ls_mbr[i]->fx_data.nicotine.poisoning_rate = _100_PERCENT;
+		ls_mbr[i]->fx_data.nicotine.lost_rate = _100_PERCENT;
+	}
 }
 
 /**/
@@ -990,7 +1024,8 @@ item_t	*init_mbr_item(
 		break;
 	case ITEM_KIND_LIGHT:
 		tab_n = get_light_tab_n( get_light_tab(
-				sub_kind->light ) );
+				sub_kind->light,
+				sub_kind->wish_level ) );
 		break;
 	case ITEM_KIND_SPIKE:
 		break;
@@ -1279,19 +1314,29 @@ void	new_mbr(
 
 	mbr->fx_data.bless = 0;
 
+	mbr->contribution = 0;
+
 	mbr->mnstr_kind = MNSTR_KIND_NULL;
 	mbr->mnstr_tab = NULL;
 
 	mbr->work.flg = FLG_NULL;
+	mbr->work.hyper_boss = HYPER_BOSS_NULL;
+	mbr->work.flg_hyper_boss = FALSE;
 	mbr->work.caught = 0;
 	mbr->work.catch_chr = NULL;
 	mbr->work.link_chr = NULL;
+	mbr->work.teleport_n = 0;
+	mbr->work.teleport_party_n = 0;
+	mbr->work.flg_can_wish = TRUE;
 	mbr->work.request = NULL;
 	mbr->work.towner_dx = 0;
 	mbr->work.towner_dy = 0;
 	mbr->work.enter_door = NULL;
 	mbr->work.dismissal_turn = 0;
 	mbr->work.flg_used_energy_drain = FALSE;
+	mbr->work.sync_vfx_num = 0;
+	mbr->work.sync_vfx_x = MAP_DEL_X;
+	mbr->work.sync_vfx_y = MAP_DEL_Y;
 
 	mbr->graph_name[0] = '\0';
 
@@ -1391,7 +1436,9 @@ void	set_mbr_dir( void )
 	}
 }
 
-/**/
+/***************************************************************
+* Сޥåפ˽и
+***************************************************************/
 
 void	appear_party( void )
 {
@@ -1428,7 +1475,10 @@ void	appear_party( void )
 	redraw_all();
 }
 
-/**/
+/***************************************************************
+* Сޥåפ˽и ()
+* return : ?
+***************************************************************/
 
 bool_t	appear_party_randm( void )
 {
@@ -1471,6 +1521,11 @@ bool_t	appear_party_randm( void )
 	return FALSE;
 }
 
+/***************************************************************
+* Сޥåפ˽и (򥵡)
+* return : ?
+***************************************************************/
+
 bool_t	appear_party_srch( void )
 {
 	long	randm_x, randm_y;
@@ -1533,6 +1588,10 @@ bool_t	appear_party_srch( void )
 	return FALSE;
 }
 
+/***************************************************************
+* Сޥåפ˽и (Ū)
+***************************************************************/
+
 void	appear_party_force( void )
 {
 	long	x, y;
@@ -1547,13 +1606,80 @@ void	appear_party_force( void )
 	party.square.pre_y = party.square.y;
 
 	for( n = 0; n < MBR_MAX_N; n++ ){
-		party.mbr[n]->x = MAP_DEL_X;
-		party.mbr[n]->y = MAP_DEL_Y;
+		if( chk_flg_or( party.mbr[n]->stat,
+				FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD ) ){
+			continue;
+		}
 
 		put_chr( party.mbr[n], x, y );
 	}
 }
 
+/***************************************************************
+* Сޥåפ˽и (饹ܥ)
+***************************************************************/
+
+void	appear_party_last_boss( void )
+{
+	appear_party_map_event( MAP_EVENT_LAST_BOSS_APPEAR_PARTY );
+}
+
+/***************************************************************
+* Сޥåפ˽и (ޥåס٥Ȱ)
+* char mnr : ޥåס٥ȼʸ
+***************************************************************/
+
+void	appear_party_map_event( char mnr )
+{
+	pos_t	pos;
+	long	x, y;
+	long	n;
+
+	for( n = 0; n < MBR_MAX_N; n++ ){
+		lost_trgt( party.mbr[n] );
+		clr_chr_trgt_act( party.mbr[n], FALSE );
+	}
+
+	release_all_mbr();
+	clr_map_all_mbr();
+
+	pos = get_map_event_pos( mnr );
+	x = pos.x;
+	y = pos.y;
+
+	party.square.x = x;
+	party.square.y = y;
+	party.square.pre_x = party.square.x;
+	party.square.pre_y = party.square.y;
+
+	for( n = 0; n < MBR_MAX_N; n++ ){
+		long	nx, ny;
+
+		if( chk_flg_or( party.mbr[n]->stat,
+				FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD ) ){
+			continue;
+		}
+
+		nx = party.square.x + party.square.chr_pos[n].x;
+		ny = party.square.y + party.square.chr_pos[n].y;
+
+		party.mbr[n]->x = MAP_DEL_X;
+		party.mbr[n]->y = MAP_DEL_Y;
+
+		put_chr( party.mbr[n], nx, ny );
+	}
+
+	appear_all_pet();
+
+	next_pos_square( 0 );
+	scroll_center_appear_party();
+	set_flg_find_all_mnstr();
+
+	redraw_all();
+}
+
 /***************************************************************
 * Сߤꤿֿ
 * return : ֿ
@@ -1581,9 +1707,22 @@ long	get_party_dun_max_lev( void )
 
 void	scroll_center_appear_party( void )
 {
+	pos_t	*draw;
+	pos_t	*draw_gui;
+
 	if( chk_flg_or( get_flg_scroll(),
 			(FLG_SCROLL_CENTER | FLG_SCROLL_SMOOTH) ) ){
 		scroll_center();
+	} else {
+		scroll_half_page();
+	}
+
+	draw = get_map_draw_pos();
+	draw_gui = get_map_draw_pos_gui();
+	if( (draw != NULL) && (draw_gui != NULL) ){
+		draw_gui->x = draw->x;
+		draw_gui->y = draw->y;
+		call_pcg_dun_scroll( draw_gui->x, draw_gui->y );
 	}
 }
 
@@ -1801,15 +1940,8 @@ void	die_mbr( mbr_t *mbr, bool_t flg_chk_game_over )
 
 		break;
 	}
-	if( i >= MBR_MAX_N ){
-		print_msg( FLG_MSG_NO_MORE_PREV, MSG_S, MSG_GAME_OVER );
-		while( 1 ){
-			switch( get_key( g_flg_clr_key_buf ) ){
-			case CASE_EXIT:
-				exit_game( EXIT_SUCCESS );
-			}
-		}
-	}
+	if( i >= MBR_MAX_N )
+		game_over();
 }
 
 /**/
@@ -2272,7 +2404,7 @@ void	exp_party( long exp, bool_t flg_set_max, bool_t flg_draw )
 	if( flg_draw ){
 		curs_attrset_chr( get_mbr_randm( MAP_DEL_X, MAP_DEL_Y ),
 				+1, FALSE );
-		print_msg( FLG_NULL, MSG_EXP_PARTY, exp_add );
+		print_msg( FLG_NULL, MSG_EXP_PARTY, fmt_num( exp_add ) );
 		curs_attrset_dflt();
 	}
 
@@ -2791,10 +2923,13 @@ char	*get_race_str( race_t *race )
 	str_nz_cpy( str_main, get_race_str_main( race ), RACE_STR_MAX_LEN );
 	str_nz_cpy( str_sub, get_race_str_sub( race ), RACE_STR_MAX_LEN );
 
-	if( race->sub == SUB_RACE_NULL )
-		sprintf( str_tmp, MSG_RACE_STR_PUREBRED, str_main );
-	else
-		sprintf( str_tmp, MSG_RACE_STR_MIXED, str_main, str_sub );
+	if( race->sub == SUB_RACE_NULL ){
+		sn_printf( str_tmp, RACE_STR_MAX_LEN,
+				MSG_RACE_STR_PUREBRED, str_main );
+	} else {
+		sn_printf( str_tmp, RACE_STR_MAX_LEN,
+				MSG_RACE_STR_MIXED, str_main, str_sub );
+	}
 
 	str_nz_cpy( str_name, str_tmp, RACE_STR_MAX_LEN );
 
@@ -3054,6 +3189,32 @@ bool_t	cmp_class( class_t *org, class_t *dst )
 
 /**/
 
+void	arrange_class_1( long class_n, long d )
+{
+	mbr_t	**ls_mbr = get_ls_mbr();
+	class_t	*p = get_class_tab();
+	class_t	tmp;
+	long	i;
+
+	if( (class_n + d) < 0 )
+		return;
+	if( (class_n + d) >= CLASS_MAX_N )
+		return;
+
+	tmp = p[class_n];
+	p[class_n] = p[class_n + d];
+	p[class_n + d] = tmp;
+
+	for( i = 0; i < LS_MBR_MAX_N; i++ ){
+		if( ls_mbr[i]->class_n == class_n )
+			ls_mbr[i]->class_n = (class_n + d);
+		else if( ls_mbr[i]->class_n == (class_n + d) )
+			ls_mbr[i]->class_n = class_n;
+	}
+}
+
+/**/
+
 void	arrange_class( long class_n, long d )
 {
 	mbr_t	**ls_mbr = get_ls_mbr();
@@ -3114,3 +3275,38 @@ void	set_square_mbr( square_t *sq )
 
 /**/
 
+void	get_square_pos( long *x, long *y )
+{
+	if( x != NULL )
+		*x = party.square.x;
+	if( y != NULL )
+		*y = party.square.y;
+}
+
+/**/
+
+void	set_square_pos( long x, long y )
+{
+	if( !clip_pos( x, y ) )
+		return;
+
+	party.square.x = x;
+	party.square.y = y;
+}
+
+/**/
+
+void	unset_square_pos( void )
+{
+	if( !clip_pos( party.square.pre_x, party.square.pre_y ) )
+		return;
+
+	party.square.x = party.square.pre_x;
+	party.square.y = party.square.pre_y;
+	party.square.dir = party.square.pre_dir;
+
+	set_mbr_dir();
+}
+
+/**/
+
