@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: pet.c,v 1.59 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: pet.c,v 1.60 2008/04/16 16:12:34 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -69,6 +69,14 @@ void	init_pet( void )
 		init_pet_debug();
 }
 
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_pet( void )
+{
+}
+
 /***************************************************************
 * ǥХåΥڥåȤν
 ***************************************************************/
