@@ -13,7 +13,7 @@ char	*cb_menu_temple_cure_status( menu_t **pp, menu_t **dflt );
 char	*cb_menu_temple_resurrection( menu_t **pp, menu_t **dflt );
 char	*cb_menu_temple_contribution( menu_t **pp, menu_t **dflt );
 char	*menu_restaurant( void );
-bool_t	chk_shop_order( shop_n_t g_shop_n );
+bool_t	chk_shop_order( shop_n_t shop_n );
 char	*menu_order( void );
 char	*menu_order_sel_mbr( void );
 char	*cb_menu_order_sel( menu_t **pp, menu_t **dflt );
@@ -71,8 +71,11 @@ char	*cb_menu_bar_edit_mbr_init( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_edit_mbr_sub( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_data_form( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_data_edit( menu_t **pp, menu_t **dflt );
+char	*cb_menu_bar_arrange_up( menu_t **pp, menu_t **dflt );
+char	*cb_menu_bar_arrange_down( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_top( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_bottom( menu_t **pp, menu_t **dflt );
+char	*cb_menu_bar_arrange_1( long d );
 char	*cb_menu_bar_arrange( long d );
 char	*cb_menu_bar_new_mbr( menu_t **pp, menu_t **dflt );
 bool_t	exec_menu_new_mbr( void );
@@ -81,6 +84,8 @@ char	*cb_menu_bar_remove_mbr_sub( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_edit_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_edit_class_sub( menu_t **pp, menu_t **dflt );
 char	*menu_edit_class( long n );
+char	*cb_menu_bar_arrange_up_class( menu_t **pp, menu_t **dflt );
+char	*cb_menu_bar_arrange_down_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_top_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_bottom_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_new_class( menu_t **pp, menu_t **dflt );
@@ -125,6 +130,8 @@ char	*cb_menu_auto_sale( menu_t **pp, menu_t **dflt );
 bool_t	set_menu_auto_sale( menu_t *ls, mbr_t *mbr );
 char	*cb_menu_auto_sale_chk( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_sale_exec( menu_t **pp, menu_t **dflt );
+shop_n_t	get_cur_shop_n( void );
+void	set_cur_shop_n( shop_n_t n );
 
 #ifdef __cplusplus
 }
