@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: tmenu-tab.h,v 1.22 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: tmenu-tab.h,v 1.24 2008/09/21 00:58:16 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	TMENU_TAB_H
@@ -1655,6 +1655,24 @@ menu_t	ls_menu_bar_edit_mbr_sub[] = {
 		cb_menu_bar_data_edit, NULL,
 		FLG_NULL,
 	},
+	{
+		"/edit member/arrange up",
+		"<Item>",
+		'U',
+		"",
+		N_MSG_MENU_BAR_ARRANGE_UP, "",
+		cb_menu_bar_arrange_up, NULL,
+		FLG_NULL,
+	},
+	{
+		"/edit member/arrange down",
+		"<Item>",
+		'D',
+		"",
+		N_MSG_MENU_BAR_ARRANGE_DOWN, "",
+		cb_menu_bar_arrange_down, NULL,
+		FLG_NULL,
+	},
 	{
 		"/edit member/arrange top",
 		"<Item>",
@@ -1712,6 +1730,24 @@ menu_t	ls_menu_bar_sel_edit_class[] = {
 		cb_menu_bar_edit_class_sub, NULL,
 		FLG_NULL,
 	},
+	{
+		"/select edit class/arrange up",
+		"<Item>",
+		'U',
+		"",
+		N_MSG_MENU_BAR_ARRANGE_UP_CLASS, "",
+		cb_menu_bar_arrange_up_class, NULL,
+		FLG_NULL,
+	},
+	{
+		"/select edit class/arrange down",
+		"<Item>",
+		'D',
+		"",
+		N_MSG_MENU_BAR_ARRANGE_DOWN_CLASS, "",
+		cb_menu_bar_arrange_down_class, NULL,
+		FLG_NULL,
+	},
 	{
 		"/select edit class/arrange top",
 		"<Item>",
