@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: tmenu.c,v 1.118 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: tmenu.c,v 1.127 2008/09/21 00:58:16 bowkenken Exp $
 ***************************************************************/
 
 #define	TMENU_C
@@ -73,7 +73,11 @@ static menu_t	*g_navi_dflt;
 
 /**/
 
-void	init_tmenu( void )
+static edit_class_t	static_exec_edit_class__n;
+
+/**/
+
+void	init_town_menu( void )
 {
 	party_t	*pty = get_party();
 	mbr_t	**ls_mbr = get_ls_mbr();
@@ -86,6 +90,33 @@ void	init_tmenu( void )
 	g_navi_dflt = NULL;
 }
 
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_town_menu( void )
+{
+	party_t	*pty = get_party();
+	mbr_t	**ls_mbr = get_ls_mbr();
+	long	i;
+
+	g_sel_mbr = pty->mbr[0];
+	g_sel_ls_mbr = ls_mbr[0];
+	g_shop_n = SHOP_N_NULL;
+	g_deal_kind = DEAL_KIND_BUY;
+	g_sel_art = NULL;
+	g_sel_class_n = 0;
+
+	for( i = 0; i < MBR_MAX_N; i++ )
+		g_mbr_order[i] = NULL;
+
+	g_navi_dflt = NULL;
+
+	/**/
+
+	static_exec_edit_class__n = EDIT_CLASS_NEXT;
+}
+
 /**/
 
 char	*menu_shop_sel_mbr( shop_n_t shop_n, bool_t flg_pet )
@@ -104,7 +135,7 @@ char	*menu_shop_sel_mbr( shop_n_t shop_n, bool_t flg_pet )
 		char	ttl[MENU_MSG_MAX_LEN + 1];
 		char	*str_goto;
 
-		sprintf( ttl, MSG_MENU_SHOP_SEL_MBR,
+		sn_printf( ttl, MENU_MSG_MAX_LEN, MSG_MENU_SHOP_SEL_MBR,
 				get_shop_name( g_shop_n ) );
 
 		str_goto = exec_menu_sel_mbr(
@@ -160,22 +191,22 @@ char	*menu_shop_sel_mbr( shop_n_t shop_n, bool_t flg_pet )
 
 char	*cb_menu_temple_init( menu_t **pp, menu_t **dflt )
 {
-	long	len;
+	long	len, max_len;
 	menu_t	*p;
 	gold_t	price;
 
-	len = 0;
+	max_len = 1;
 	for( p = *pp; p->path[0] != '\0'; p++ ){
 		long	tmp_len;
 
 		tmp_len = str_len_std( MSG( p->n_msg ) );
-		len = max_l( len, tmp_len );
+		max_len = max_l( max_len, tmp_len );
 	}
 
 	p = srch_menu( *pp, "title" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_TTL_MENU_TEMPLE,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_TEMPLE,
 			get_shop_name( SHOP_N_TEMPLE ),
 			g_sel_mbr->name, fmt_gold( g_sel_mbr->gold ) );
 
@@ -183,21 +214,24 @@ char	*cb_menu_temple_init( menu_t **pp, menu_t **dflt )
 	p = srch_menu( *pp, "remove curse" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_FMT_MENU_TEMPLE,
+	len = get_pading_len( MSG( p->n_msg ), max_len );
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_FMT_MENU_TEMPLE,
 			len, MSG( p->n_msg ), fmt_gold( price ) );
 
 	price = get_price_temple_cure_status( g_sel_mbr );
 	p = srch_menu( *pp, "cure status" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_FMT_MENU_TEMPLE,
+	len = get_pading_len( MSG( p->n_msg ), max_len );
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_FMT_MENU_TEMPLE,
 			len, MSG( p->n_msg ), fmt_gold( price ) );
 
 	price = get_price_temple_resurrection( g_sel_mbr );
 	p = srch_menu( *pp, "resurrection" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_FMT_MENU_TEMPLE,
+	len = get_pading_len( MSG( p->n_msg ), max_len );
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_FMT_MENU_TEMPLE,
 			len, MSG( p->n_msg ), fmt_gold( price ) );
 
 	return NULL;
@@ -233,6 +267,8 @@ char	*cb_menu_temple_remove_curse( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_TEMPLE );
+
 	if( remove_curse( NULL, g_sel_mbr, 0 ) ){
 		add_chr_gold( chr_pay, -price );
 
@@ -278,6 +314,8 @@ char	*cb_menu_temple_cure_status( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_TEMPLE );
+
 	if( fx_cure_all_stat( g_sel_mbr ) ){
 		add_chr_gold( chr_pay, -price );
 
@@ -328,6 +366,8 @@ char	*cb_menu_temple_resurrection( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_TEMPLE );
+
 	if( !chk_flg( g_sel_mbr->stat, FLG_STAT_DEAD ) ){
 		print_msg( FLG_MSG_ERR,
 				MSG_MENU_RESURRECTION_ERR,
@@ -383,6 +423,8 @@ char	*cb_menu_temple_contribution( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_TEMPLE );
+
 	add_chr_gold( chr_pay, -price );
 	print_msg( FLG_NULL, MSG_MENU_CONTRIBUTION,
 			g_sel_mbr->name, fmt_gold( price ) );
@@ -402,11 +444,11 @@ char	*menu_restaurant( void )
 
 /**/
 
-bool_t	chk_shop_order( shop_n_t g_shop_n )
+bool_t	chk_shop_order( shop_n_t shop_n )
 {
-	if( g_shop_n == SHOP_N_BAR )
+	if( shop_n == SHOP_N_BAR )
 		return TRUE;
-	if( g_shop_n == SHOP_N_RESTAURANT )
+	if( shop_n == SHOP_N_RESTAURANT )
 		return TRUE;
 
 	return FALSE;
@@ -489,7 +531,7 @@ char	*menu_order_sel_mbr( void )
 		if( (g_sel_mbr != NULL) && (g_sel_mbr->mbr_n == i) )
 			dflt = &(p[k]);
 
-		sprintf( p[k].path, "/order/%ld", i );
+		sn_printf( p[k].path, MENU_PATH_MAX_LEN, "/order/%ld", i );
 		str_nz_cpy( p[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
 		p[k].accel_key = (char)('A' + i);
 		p[k].short_key[0] = '\0';
@@ -497,7 +539,8 @@ char	*menu_order_sel_mbr( void )
 			name = MSG_MENU_ORDER_NULL;
 		else
 			name = g_mbr_order[i]->name;
-		sprintf( p[k].msg, MSG_MENU_ORDER_MBR, mbr[i]->name, name );
+		sn_printf( p[k].msg, MENU_MSG_MAX_LEN,
+				MSG_MENU_ORDER_MBR, mbr[i]->name, name );
 		p[k].call_back = cb_menu_order_sel;
 		p[k].arg = mbr[i];
 		p[k].flg = FLG_NULL;
@@ -723,7 +766,8 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 					MENU_KIND_MAX_LEN );
 			ls[k].accel_key = '\0';
 			ls[k].short_key[0] = '\0';
-			sprintf( ls[k].msg,
+			sn_printf( ls[k].msg,
+					MENU_MSG_MAX_LEN,
 					MSG_FMT_MENU_SHOP,
 					get_shop_name( shop_n ),
 					mbr->name, buy_sale,
@@ -746,7 +790,8 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 		if( art == g_sel_art )
 			dflt = &(ls[k]);
 
-		sprintf( ls[k].path, "/shop/%ld", key_n );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/shop/%ld", key_n );
 		str_nz_cpy( ls[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = (char)('A' + key_n);
 		ls[k].short_key[0] = '\0';
@@ -772,7 +817,8 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 				MENU_KIND_MAX_LEN );
 		ls[k].accel_key = '\0';
 		ls[k].short_key[0] = '\0';
-		sprintf( ls[k].msg,
+		sn_printf( ls[k].msg,
+				MENU_MSG_MAX_LEN,
 				MSG_FMT_MENU_SHOP,
 				get_shop_name( shop_n ),
 				mbr->name, buy_sale,
@@ -793,7 +839,8 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 		if( ls[i].arg == NULL )
 			continue;
 
-		sprintf( ls[i].msg,
+		sn_printf( ls[i].msg,
+				MENU_MSG_MAX_LEN,
 				MSG_FMT_ART_NAME,
 				max_len, ls_art[j]->art_name,
 				fmt_gold( get_price(
@@ -1584,7 +1631,8 @@ char	*cb_menu_bar_form( menu_t **pp, menu_t **dflt )
 				break;
 			}
 
-			sprintf( p[k].path, "/bar form/%ld", ls_mbr_n );
+			sn_printf( p[k].path, MENU_PATH_MAX_LEN,
+					"/bar form/%ld", ls_mbr_n );
 			str_nz_cpy( p[k].kind, "<CheckItem>",
 					MENU_KIND_MAX_LEN );
 			p[k].accel_key = (char)('A' + j);
@@ -1679,7 +1727,8 @@ char	*cb_menu_bar_form_sub_init( menu_t **pp, menu_t **dflt )
 
 	clr_flg_menu( *pp, NULL, FLG_MENU_ALL );
 
-	sprintf( sub_path, "%ld", g_sel_ls_mbr->ls_mbr_n );
+	sn_printf( sub_path, MENU_PATH_MAX_LEN,
+			"%ld", g_sel_ls_mbr->ls_mbr_n );
 
 	if( g_sel_ls_mbr->mbr_n == MBR_N_NOT_JOIN ){
 		set_chk_off_menu( ls_menu_bar_form, sub_path );
@@ -1731,12 +1780,12 @@ char	*cb_menu_bar_join( menu_t **pp, menu_t **dflt )
 
 	n = 0;
 	for( i = 0; i < MBR_MAX_N; i++ ){
-		sprintf( p[k].path, "/join/%ld", i );
+		sn_printf( p[k].path, MENU_PATH_MAX_LEN, "/join/%ld", i );
 		str_nz_cpy( p[k].kind, "<Item>",
 				MENU_KIND_MAX_LEN );
 		p[k].accel_key = (char)('A' + i);
 		p[k].short_key[0] = '\0';
-		sprintf( p[k].msg, "%ld", i + 1 );
+		sn_printf( p[k].msg, MENU_MSG_MAX_LEN, "%ld", i + 1 );
 		p[k].call_back = cb_menu_bar_join_sub;
 		p[k].arg = (void *)i;
 		p[k].flg = FLG_NULL;
@@ -1794,7 +1843,8 @@ char	*cb_menu_bar_join_sub( menu_t **pp, menu_t **dflt )
 	mbr_n = (long)((*pp)->arg);
 	join_mbr( g_sel_ls_mbr->ls_mbr_n, mbr_n );
 
-	sprintf( sub_path, "%ld", g_sel_ls_mbr->ls_mbr_n );
+	sn_printf( sub_path, MENU_PATH_MAX_LEN,
+			"%ld", g_sel_ls_mbr->ls_mbr_n );
 	set_chk_off_menu( ls_menu_bar_form, sub_path );
 
 	next_g_sel_ls_mbr();
@@ -1813,7 +1863,8 @@ char	*cb_menu_bar_not_join( menu_t **pp, menu_t **dflt )
 
 	not_join_mbr( g_sel_ls_mbr->ls_mbr_n, g_sel_ls_mbr->mbr_n );
 
-	sprintf( sub_path, "%ld", g_sel_ls_mbr->ls_mbr_n );
+	sn_printf( sub_path, MENU_PATH_MAX_LEN,
+			"%ld", g_sel_ls_mbr->ls_mbr_n );
 	set_chk_on_menu( ls_menu_bar_form, sub_path );
 
 	next_g_sel_ls_mbr();
@@ -1892,7 +1943,8 @@ char	*cb_menu_bar_edit_mbr( menu_t **pp, menu_t **dflt )
 				break;
 			}
 
-			sprintf( p[k].path, "/edit member/%ld",
+			sn_printf( p[k].path, MENU_PATH_MAX_LEN,
+					"/edit member/%ld",
 					ls_mbr_n );
 			str_nz_cpy( p[k].kind, "<Branch>",
 					MENU_KIND_MAX_LEN );
@@ -1967,7 +2019,8 @@ char	*cb_menu_bar_edit_mbr_sub( menu_t **pp, menu_t **dflt )
 	p = srch_menu( *pp, "title" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_TTL_MENU_BAR_EDIT_MBR_SUB,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN,
+			MSG_TTL_MENU_BAR_EDIT_MBR_SUB,
 			g_sel_ls_mbr->name );
 
 	return NULL;
@@ -1993,6 +2046,20 @@ char	*cb_menu_bar_data_edit( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_bar_arrange_up( menu_t **pp, menu_t **dflt )
+{
+	return cb_menu_bar_arrange_1( -1 );
+}
+
+/**/
+
+char	*cb_menu_bar_arrange_down( menu_t **pp, menu_t **dflt )
+{
+	return cb_menu_bar_arrange_1( +1 );
+}
+
+/**/
+
 char	*cb_menu_bar_arrange_top( menu_t **pp, menu_t **dflt )
 {
 	return cb_menu_bar_arrange( -1 );
@@ -2007,6 +2074,44 @@ char	*cb_menu_bar_arrange_bottom( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_bar_arrange_1( long d )
+{
+	long	trgt_ls_mbr_n;
+	mbr_t	**ls_mbr, *tmp_mbr;
+
+	if( g_sel_ls_mbr == NULL )
+		return STR_MENU_ERR;
+
+	ls_mbr = get_ls_mbr();
+	trgt_ls_mbr_n = g_sel_ls_mbr->ls_mbr_n;
+	if( (trgt_ls_mbr_n + d) < 0 )
+		return "/bar/form/edit member/edit member";
+	if( (trgt_ls_mbr_n + d) >= LS_MBR_MAX_N )
+		return "/bar/form/edit member/edit member";
+	if( chk_flg( ls_mbr[trgt_ls_mbr_n + d]->stat,
+			FLG_STAT_NOT_EXIST ) ){
+		return "/bar/form/edit member/edit member";
+	}
+
+	arrange_cast_tab_1( trgt_ls_mbr_n, d );
+	arrange_auto_mark_mbr_1( trgt_ls_mbr_n, d );
+
+	/**/
+
+	tmp_mbr = ls_mbr[trgt_ls_mbr_n];
+
+	ls_mbr[trgt_ls_mbr_n] = ls_mbr[trgt_ls_mbr_n + d];
+	ls_mbr[trgt_ls_mbr_n]->ls_mbr_n = trgt_ls_mbr_n;
+
+	ls_mbr[trgt_ls_mbr_n + d] = tmp_mbr;
+	ls_mbr[trgt_ls_mbr_n + d]->ls_mbr_n = trgt_ls_mbr_n + d;
+
+	g_sel_ls_mbr = ls_mbr[0];
+	return "/bar/form/edit member/edit member";
+}
+
+/**/
+
 char	*cb_menu_bar_arrange( long d )
 {
 	long	trgt_ls_mbr_n;
@@ -2163,7 +2268,8 @@ char	*cb_menu_bar_remove_mbr( menu_t **pp, menu_t **dflt )
 				break;
 			}
 
-			sprintf( p[k].path, "/bar remove member/%ld",
+			sn_printf( p[k].path, MENU_PATH_MAX_LEN,
+					"/bar remove member/%ld",
 					ls_mbr_n );
 			str_nz_cpy( p[k].kind, "<OpenWinItem>",
 					MENU_KIND_MAX_LEN );
@@ -2305,6 +2411,24 @@ char	*menu_edit_class( long n )
 
 /**/
 
+char	*cb_menu_bar_arrange_up_class( menu_t **pp, menu_t **dflt )
+{
+	arrange_class_1( g_sel_class_n, -1 );
+
+	return NULL;
+}
+
+/**/
+
+char	*cb_menu_bar_arrange_down_class( menu_t **pp, menu_t **dflt )
+{
+	arrange_class_1( g_sel_class_n, +1 );
+
+	return NULL;
+}
+
+/**/
+
 char	*cb_menu_bar_arrange_top_class( menu_t **pp, menu_t **dflt )
 {
 	arrange_class( g_sel_class_n, -1 );
@@ -2371,11 +2495,13 @@ char	*menu_new_class( long n )
 
 long	exec_edit_class( class_t *org, bool_t flg_new )
 {
-	static edit_class_t	n = EDIT_CLASS_NEXT;
+	edit_class_t	n;
 	edit_class_t	prev_n;
 	class_t	edit, *work;
 	bool_t	flg_sel_add;
 
+	n = static_exec_edit_class__n;
+
 	if( org == NULL )
 		return 0;
 
@@ -2397,11 +2523,16 @@ long	exec_edit_class( class_t *org, bool_t flg_new )
 		draw_edit_class( org, work, n, TRUE, flg_sel_add );
 		prev_n = n;
 
+		call_gui_class_edit_draw( work );
+
 		switch( get_key( g_flg_clr_key_buf_menu ) ){
 		case CASE_CANCEL:
 		case CASE_EXEC:
-			if( exec_menu_ask_edit_class( org, work, flg_new ) )
+			if( exec_menu_ask_edit_class(
+					org, work, flg_new ) ){
+				call_gui_class_edit_wipe();
 				return 0;
+			}
 			draw_edit_class_all( org, work, n, flg_sel_add );
 			break;
 		case CASE_ENTER:
@@ -2683,7 +2814,7 @@ char	*menu_inn( void )
 	p = srch_menu( ls_menu_inn, "stay" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_MENU_INN_STAY,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_MENU_INN_STAY,
 			fmt_gold( get_price_inn( FALSE ) ) );
 
 	/* ٷ */
@@ -2691,7 +2822,7 @@ char	*menu_inn( void )
 	p = srch_menu( ls_menu_inn, "short stay" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_MENU_INN_SHORT_STAY,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_MENU_INN_SHORT_STAY,
 			fmt_gold( get_price_inn( TRUE ) ) );
 
 	/* Ͼǰ */
@@ -2699,7 +2830,7 @@ char	*menu_inn( void )
 	p = srch_menu( ls_menu_inn, "stay in mews" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_MENU_INN_STAY_IN_MEWS,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_MENU_INN_STAY_IN_MEWS,
 			fmt_gold( 0 ) );
 
 	/**/
@@ -2721,19 +2852,36 @@ char	*cb_menu_inn_stay( menu_t **pp, menu_t **dflt )
 	price = get_price_inn( flg_short );
 
 	flg_pay = TRUE;
-	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ )
+	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ ){
+		if( chk_flg_or( pty->mbr[mbr_n]->stat, 0
+				| FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_STONE ) ){
+			continue;
+		}
+
 		if( !chk_pay( pty->mbr[mbr_n], price ) )
 			flg_pay = FALSE;
+	}
 
 	if( !flg_pay )
 		return STR_MENU_CANCEL;
 
-	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ )
-		add_chr_gold( pty->mbr[mbr_n], -price );
+	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ ){
+		if( chk_flg_or( pty->mbr[mbr_n]->stat, 0
+				| FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_STONE ) ){
+			continue;
+		}
 
+		add_chr_gold( pty->mbr[mbr_n], -price );
+	}
+
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_INN );
 	stay_inn( flg_short, FALSE );
 
-	return NULL;
+	return STR_MENU_CANCEL;
 }
 
 /**/
@@ -2747,9 +2895,10 @@ char	*cb_menu_inn_short_stay( menu_t **pp, menu_t **dflt )
 
 char	*cb_menu_inn_stay_in_mews( menu_t **pp, menu_t **dflt )
 {
+	call_game_music_play( MUSIC_KIND_EFFECT, ME_KIND_INN );
 	stay_inn( FALSE, TRUE );
 
-	return NULL;
+	return STR_MENU_CANCEL;
 }
 
 /**/
@@ -2793,7 +2942,7 @@ char	*cb_menu_magic_guild_init( menu_t **pp, menu_t **dflt )
 	p = srch_menu( *pp, "title" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_TTL_MENU_MAGIC_GUILD,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_MAGIC_GUILD,
 			get_shop_name( SHOP_N_MAGIC ),
 			g_sel_mbr->name,
 			fmt_gold( g_sel_mbr->gold ) );
@@ -2826,11 +2975,13 @@ char	*cb_menu_magic_guild_init( menu_t **pp, menu_t **dflt )
 		tab = get_spell_tab( kind );
 		if( tab == NULL ){
 			gold = 0;
-			sprintf( ls[i].msg, MSG_FMT_MENU_MAGIC_GUILD,
+			sn_printf( ls[i].msg, MENU_MSG_MAX_LEN,
+					MSG_FMT_MENU_MAGIC_GUILD,
 					"", 0, fmt_gold( 0 ) );
 		} else {
 			gold = get_price_spell( kind );
-			sprintf( ls[i].msg, MSG_FMT_MENU_MAGIC_GUILD,
+			sn_printf( ls[i].msg, MENU_MSG_MAX_LEN,
+					MSG_FMT_MENU_MAGIC_GUILD,
 					tab->name, tab->lev,
 					fmt_gold( gold ) );
 		}
@@ -2959,7 +3110,7 @@ char	*cb_menu_pet_shop_init( menu_t **pp, menu_t **dflt )
 	p = srch_menu( *pp, "title" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_TTL_MENU_PET_SHOP,
+	sn_printf( p->msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_PET_SHOP,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name,
 			fmt_gold( g_sel_mbr->gold ) );
@@ -3001,7 +3152,7 @@ char	*cb_menu_pet_shop_take( menu_t **pp, menu_t **dflt )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_TAKE,
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_PET_SHOP_TAKE,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name );
 	ls[k].call_back = NULL;
@@ -3017,7 +3168,8 @@ char	*cb_menu_pet_shop_take( menu_t **pp, menu_t **dflt )
 		if( !chk_can_join_pet( ls_pet[i] ) )
 			continue;
 
-		sprintf( ls[k].path, "/pet shop take/%ld", (long)(i + 1) );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/pet shop take/%ld", (long)(i + 1) );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
@@ -3114,7 +3266,7 @@ char	*cb_menu_pet_shop_leave( menu_t **pp, menu_t **dflt )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_LEAVE,
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_PET_SHOP_LEAVE,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name );
 	ls[k].call_back = NULL;
@@ -3130,7 +3282,8 @@ char	*cb_menu_pet_shop_leave( menu_t **pp, menu_t **dflt )
 		if( !chk_can_not_join_pet( party_pet[i] ) )
 			continue;
 
-		sprintf( ls[k].path, "/pet shop leave/%ld", (long)(i + 1) );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/pet shop leave/%ld", (long)(i + 1) );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
@@ -3227,7 +3380,7 @@ char	*cb_menu_pet_shop_name( menu_t **pp, menu_t **dflt )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_NAME,
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_PET_SHOP_NAME,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name );
 	ls[k].call_back = NULL;
@@ -3246,12 +3399,14 @@ char	*cb_menu_pet_shop_name( menu_t **pp, menu_t **dflt )
 		else
 			mnstr_name = ls_pet[i]->mnstr_tab->name;
 
-		sprintf( ls[k].path, "/pet shop name/%ld", (long)(i + 1) );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/pet shop name/%ld", (long)(i + 1) );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
 		ls[k].n_msg = N_MSG_NULL,
-		sprintf( ls[k].msg, MSG_MENU_PET_SHOP_NAME_LS,
+		sn_printf( ls[k].msg, MENU_MSG_MAX_LEN,
+				MSG_MENU_PET_SHOP_NAME_LS,
 				ls_pet[i]->name, mnstr_name );
 		ls[k].call_back = cb_menu_pet_shop_name_sel;
 		ls[k].arg = (void *)(ls_pet[i]);
@@ -3347,7 +3502,8 @@ char	*cb_menu_pet_shop_remove( menu_t **pp, menu_t **dflt )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_REMOVE,
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN,
+			MSG_TTL_MENU_PET_SHOP_REMOVE,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name );
 	ls[k].call_back = NULL;
@@ -3361,7 +3517,8 @@ char	*cb_menu_pet_shop_remove( menu_t **pp, menu_t **dflt )
 		if( chk_flg( ls_pet[i]->stat, FLG_STAT_NOT_EXIST ) )
 			continue;
 
-		sprintf( ls[k].path, "/pet shop remove/%ld", (long)(i + 1) );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/pet shop remove/%ld", (long)(i + 1) );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
@@ -3475,7 +3632,7 @@ char	*cb_menu_pet_shop_buy( menu_t **pp, menu_t **dflt )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP,
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN, MSG_TTL_MENU_PET_SHOP,
 			get_shop_name( SHOP_N_PET_SHOP ),
 			g_sel_mbr->name,
 			fmt_gold( g_sel_mbr->gold ) );
@@ -3506,12 +3663,14 @@ char	*cb_menu_pet_shop_buy( menu_t **pp, menu_t **dflt )
 			key = 'A';
 		}
 
-		sprintf( ls[k].path, "/pet shop buy/%ld", (long)(i + 1) );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/pet shop buy/%ld", (long)(i + 1) );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
 		ls[k].n_msg = N_MSG_NULL,
-		sprintf( ls[k].msg, MSG_FMT_MENU_PET_SHOP,
+		sn_printf( ls[k].msg, MENU_MSG_MAX_LEN,
+				MSG_FMT_MENU_PET_SHOP,
 				get_mnstr_name( art_ls_pet_shop[i].kind ),
 				fmt_gold( art_ls_pet_shop[i].price ) );
 		ls[k].call_back = cb_menu_pet_shop_buy_sel;
@@ -3647,7 +3806,8 @@ char	*cb_menu_navi_init( menu_t **pp, menu_t **dflt )
 		n = (shop_n_t)i;
 		mnr = get_shop_mnr_face( n );
 
-		sprintf( ls[k].path, "/navi/%c", (char)mnr );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/navi/%c", (char)mnr );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = toupper( mnr );
 		ls[k].short_key[0] = '\0';
@@ -3753,7 +3913,8 @@ bool_t	set_menu_auto_sale( menu_t *ls, mbr_t *mbr )
 	ls[k].accel_key = '\0';
 	ls[k].short_key[0] = '\0';
 	ls[k].n_msg = N_MSG_NULL;
-	sprintf( ls[k].msg, MSG_TTL_MENU_AUTO_SALE, mbr->name );
+	sn_printf( ls[k].msg, MENU_MSG_MAX_LEN,
+			MSG_TTL_MENU_AUTO_SALE, mbr->name );
 	ls[k].call_back = NULL;
 	ls[k].arg = NULL;
 	ls[k].flg = FLG_NULL;
@@ -3789,12 +3950,14 @@ bool_t	set_menu_auto_sale( menu_t *ls, mbr_t *mbr )
 				item, DEAL_KIND_SALE_ITEM );
 		price = get_price( shop_n, item, DEAL_KIND_SALE_ITEM );
 
-		sprintf( ls[k].path, "/auto sale/%ld", i );
+		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
+				"/auto sale/%ld", i );
 		str_nz_cpy( ls[k].kind, "<CheckItem>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = toupper( (char)('A' + i) );
 		ls[k].short_key[0] = '\0';
 		ls[k].n_msg = N_MSG_NULL,
-		sprintf( ls[k].msg,
+		sn_printf( ls[k].msg,
+				MENU_MSG_MAX_LEN,
 				MSG_FMT_ART_NAME,
 				max_len,
 				item->art_name,
@@ -3881,3 +4044,17 @@ char	*cb_menu_auto_sale_exec( menu_t **pp, menu_t **dflt )
 
 /**/
 
+shop_n_t	get_cur_shop_n( void )
+{
+	return g_shop_n;
+}
+
+/**/
+
+void	set_cur_shop_n( shop_n_t n )
+{
+	g_shop_n = n;
+}
+
+/**/
+
