@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,17 +25,23 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: town.c,v 1.128 2007/06/11 08:25:36 bowkenken Exp $
+* $Id: town.c,v 1.140 2008/10/02 09:21:05 bowkenken Exp $
 ***************************************************************/
 
 #define	TOWN_C
 #include	"inc.h"
 
-/**/
+/***************************************************************
+* 
+***************************************************************/
 
 #define	TOWNER_AVE_N	64
 
-/**/
+#define	FLOOR_IS_FIND_NIGHT	1
+
+/***************************************************************
+* ŹαĶȻ
+***************************************************************/
 
 #define	OPEN_TURN_BAR	( 0 * TURN_PER_HOUR)
 #define	CLOSE_TURN_BAR	(24 * TURN_PER_HOUR)
@@ -64,7 +70,9 @@
 #define	OPEN_TURN_PET_SHOP	( 7 * TURN_PER_HOUR)
 #define	CLOSE_TURN_PET_SHOP	(19 * TURN_PER_HOUR)
 
-/**/
+/***************************************************************
+* ʪΥƥζµܤγؤ˴
+***************************************************************/
 
 #define	ART_LEVEL_WPN	-20
 #define	ART_LEVEL_ARMOR	-10
@@ -82,13 +90,17 @@
 #define	ART_LEVEL_DISPOSAL	-10
 #define	ART_LEVEL_SPIKE	-10
 
-/**/
+/***************************************************************
+* ʪ
+***************************************************************/
 
 #define	PRICE_NOT_IDENTIFIED_RATE	((rate_t)80)
 
 #define	CHARGE_STAY_INN_STAY	((gold_t)50)
 #define	CHARGE_STAY_INN_SHORT_STAY	((gold_t)50)
 
+/*  */
+
 #define	PRICE_WPN_MIN	50
 #define	PRICE_WPN_RATE	((rate_t)10)
 #define	PRICE_WPN_BASE	-3000
@@ -118,7 +130,7 @@
 #define	PRICE_ARW_DIV	500
 #define	PRICE_WPN_NOT_SPEND	500
 
-/**/
+/*  */
 
 #define	PRICE_ARW_ADD_HIT_MUL	1
 #define	PRICE_ARW_ADD_CRTCL_MUL	1
@@ -128,7 +140,7 @@
 
 #define	PRICE_ARW_SLAYER	10
 
-/**/
+/* ɶ */
 
 #define	PRICE_ARMOR_RATE	((rate_t)4)
 #define	PRICE_ARMOR_BASE	0
@@ -149,20 +161,20 @@
 #define	PRICE_ARMOR_ADD_CRTCL_BASE	0
 #define	PRICE_ARMOR_ADD_AC_BASE	0
 
-/**/
+/* ڴ */
 
 #define	PRICE_INST_BASE	1000
 #define	PRICE_INST_LIMIT	3000
 #define	PRICE_INST_SPELL	5000
 
-/**/
+/* ¾ */
 
 #define	PRICE_ANTHOLOGY	2000
 #define	PRICE_SPIKE	10
 #define	PRICE_CHEST	500
 #define	PRICE_DISPOSAL	10
 
-/**/
+/*  */
 
 #define	PRICE_TEMPLE_REMOVE_CURSE	50
 #define	PRICE_TEMPLE_CURE_STATUS_MUL	10
@@ -186,24 +198,39 @@
 
 discount_queue_t	g_discount_queue;
 
-/* ϤΤΨ */
+/***************************************************************
+* ¾
+***************************************************************/
 
+/* ϤΤΨ */
 #define	MBR_IS_MOLESTED_RATE	((rate_t)10)
 
-/**/
+/* µܤؤγʤȾ */
+#define	TOWN_STAIRS_R	1
 
+/* XPM βƩ */
 #define	STR_XPM_COL_NONE	"None"
 
+/***************************************************************
+* ƥǡơ֥
+***************************************************************/
+
 #include	"town-ptn.xpm"
 #include	"town-tab.h"
 #include	"rumor-tab.h"
 
+/***************************************************************
+* Хѿ
+***************************************************************/
+
 static town_ptn_t	town_ptn, town_ptn_trans;
 
 static long	g_morning_glow_n, g_evening_glow_n;
 static bool_t	g_flg_night;
 
-/**/
+/***************************************************************
+* 
+***************************************************************/
 
 void	init_town( void )
 {
@@ -218,7 +245,17 @@ void	init_town( void )
 	clr_discount();
 }
 
-/**/
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_town( void )
+{
+}
+
+/***************************************************************
+* īƤͼƤꥻå
+***************************************************************/
 
 void	reset_glow( void )
 {
@@ -230,7 +267,9 @@ void	reset_glow( void )
 	chk_evening_glow( get_turn() );
 }
 
-/**/
+/***************************************************************
+* ѥοơ֥
+***************************************************************/
 
 void	init_town_ptn_col_tab( void )
 {
@@ -251,7 +290,11 @@ void	init_town_ptn_col_tab( void )
 		town_ptn_col_tab_max_n = 0;
 }
 
-/**/
+/***************************************************************
+* ѥǥեȤǾ
+* town_ptn_t *ptn : ѥ
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_dflt( town_ptn_t *ptn )
 {
@@ -288,7 +331,11 @@ bool_t	set_town_ptn_dflt( town_ptn_t *ptn )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥ
+* town_ptn_t *ptn : ѥ
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_init( town_ptn_t *ptn )
 {
@@ -305,7 +352,12 @@ bool_t	set_town_ptn_init( town_ptn_t *ptn )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥΥإåɤ߹
+* town_ptn_t *ptn : ѥ
+* char *s : إåʸ
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_head( town_ptn_t *ptn, char *s )
 {
@@ -338,7 +390,12 @@ bool_t	set_town_ptn_head( town_ptn_t *ptn, char *s )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥοơ֥ɤ߹
+* town_ptn_t *ptn : ѥ
+* char *s : ơ֥ʸ (1 ʬ)
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_col( town_ptn_t *ptn, char *s )
 {
@@ -384,7 +441,11 @@ bool_t	set_town_ptn_col( town_ptn_t *ptn, char *s )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥο
+* town_ptn_t *ptn : ѥ
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_col_count( town_ptn_t *ptn )
 {
@@ -409,7 +470,13 @@ bool_t	set_town_ptn_col_count( town_ptn_t *ptn )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥΥޥåסǡɤ߹
+* town_ptn_t *ptn : ѥ
+* char *s : ޥåסǡʸ (1 ʬ)
+* long y : ޥåפ Y ɸ (Կ)
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	set_town_ptn_map( town_ptn_t *ptn, char *s, long y )
 {
@@ -436,7 +503,13 @@ bool_t	set_town_ptn_map( town_ptn_t *ptn, char *s, long y )
 	return TRUE;
 }
 
-/**/
+/***************************************************************
+* ѥΥޥåסǡ 1 饯ʸ
+* 1 ʸ 2 ʸѴ
+* town_ptn_t *dst : ѴΥѥ
+* town_ptn_t *src : ѴΥѥ
+* return : 顼̵ä
+***************************************************************/
 
 bool_t	trans_town_ptn( town_ptn_t *dst, town_ptn_t *src )
 {
@@ -506,6 +579,10 @@ void	make_town( void )
 
 	make_fence();
 
+	/* īͼꥻå */
+
+	reset_glow();
+
 	/*  */
 
 	make_streetlamp();
@@ -581,22 +658,34 @@ void	make_town( void )
 void	make_town_ptn( void )
 {
 	dun_t *dun = get_dun();
-	long	i, j;
+	long	x, y;
+	long	i;
 
-	for( i = 0; i < MAP_MAX_Y; i++ )
-		for( j = 0; j < MAP_MAX_X; j++ )
-			make_field_town( &town_ptn, j, i );
+	for( y = 0; y < MAP_MAX_Y; y++ )
+		for( x = 0; x < MAP_MAX_X; x++ )
+			make_field_town( &town_ptn, x, y );
 
-	for( i = 0; i < MAP_MAX_Y; i++ ){
-		for( j = 0; j < MAP_MAX_X; j++ ){
+	x = MAP_DEL_X;
+	y = MAP_DEL_Y;
+	for( i = 0; i < LOOP_MAX_100; i++ ){
+		if( !srch_map_obj( FACE_MJR_STAIRS_DOWN,
+				FACE_MNR_STAIRS_DOWN, &x, &y ) ){
+			break;
+		}
+
+		extend_stairs( x, y );
+	}
+
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
 			char	mjr, mnr;
 
-			mjr = dun->map.obj.mjr[i][j];
+			mjr = dun->map.obj.mjr[y][x];
 			if( mjr != FACE_MJR_DOOR_CLOSE )
 				continue;
-			mnr = dun->map.obj.mnr[i][j];
+			mnr = dun->map.obj.mnr[y][x];
 
-			make_door_town( j, i, mnr );
+			make_door_town( x, y, mnr );
 		}
 	}
 }
@@ -630,21 +719,27 @@ void	make_field_town( town_ptn_t *ptn, long x, long y )
 	switch( mjr ){
 	case FACE_MJR_NULL:
 		mjr = FACE_MJR_FLOOR;
-		mnr = FACE_MNR_FLOOR;
+		mnr = FACE_MNR_HOLE;
 		if( chk_night() ){
-			flg &= ~(FLG_MAP_OBJ_FIND);
+			if( FLOOR_IS_FIND_NIGHT )
+				flg |= FLG_MAP_OBJ_FIND;
+			else
+				flg &= ~(FLG_MAP_OBJ_FIND);
 			flg_light = FALSE;
 		}
-		flg |= FLG_MAP_OBJ_PASS;
 		break;
 	case FACE_MJR_WALL:
 		break;
 	case FACE_MJR_FLOOR:
 		if( chk_night() ){
-			flg &= ~(FLG_MAP_OBJ_FIND);
+			if( FLOOR_IS_FIND_NIGHT )
+				flg |= FLG_MAP_OBJ_FIND;
+			else
+				flg &= ~(FLG_MAP_OBJ_FIND);
 			flg_light = FALSE;
 		}
-		flg |= FLG_MAP_OBJ_PASS;
+		if( mnr != FACE_MNR_HOLE )
+			flg |= FLG_MAP_OBJ_PASS;
 		break;
 	case FACE_MJR_DOOR_OPEN:
 		mjr = FACE_MJR_DOOR_CLOSE;
@@ -671,6 +766,44 @@ void	make_field_town( town_ptn_t *ptn, long x, long y )
 
 /**/
 
+bool_t	extend_stairs( long x, long y )
+{
+	dun_t	*dun = get_dun();
+	char	mjr;
+	long	xr, yr;
+	long	xx, yy;
+
+	if( dun->map.obj.mjr[y][x] != FACE_MJR_STAIRS_UP )
+		if( dun->map.obj.mjr[y][x] != FACE_MJR_STAIRS_DOWN )
+			return FALSE;
+	if( dun->map.obj.mnr[y][x] != FACE_MNR_STAIRS_UP )
+		if( dun->map.obj.mnr[y][x] != FACE_MNR_STAIRS_DOWN )
+			return FALSE;
+
+	mjr = dun->map.obj.mjr[y][x];
+
+	for( yr = -TOWN_STAIRS_R; yr <= +TOWN_STAIRS_R; yr++ ){
+		for( xr = -TOWN_STAIRS_R; xr <= +TOWN_STAIRS_R; xr++ ){
+			xx = x + xr;
+			yy = y + yr;
+			if( !clip_pos( xx, yy ) )
+				continue;
+
+			if( dun->map.obj.mjr[yy][xx] != FACE_MJR_FLOOR )
+				continue;
+			if( dun->map.obj.mnr[yy][xx] != FACE_MNR_FLOOR )
+				continue;
+
+			dun->map.obj.mjr[yy][xx] = mjr;
+			dun->map.obj.mnr[yy][xx] = FACE_MNR_NULL;
+		}
+	}
+
+	return TRUE;
+}
+
+/**/
+
 void	make_streetlamp( void )
 {
 	dun_t	*dun = get_dun();
@@ -941,7 +1074,8 @@ bool_t	make_art_town( shop_n_t shop_n, void *art_ls )
 			return FALSE;
 		break;
 	case ITEM_KIND_LIGHT:
-		tab_n = get_light_tab_n( get_light_tab( light_kind ) );
+		tab_n = get_light_tab_n( get_light_tab(
+				light_kind, WISH_LEVEL_NULL ) );
 		if( tab_n <= -1 )
 			return TRUE;
 		item = make_item_light( ART_LEVEL_LIGHT, tab_n );
@@ -1162,10 +1296,9 @@ bool_t	chk_shop_face_ls( void )
 bool_t	appear_party_town( void )
 {
 	dun_t *dun;
-	long	i, j;
-	bool_t	flg_find;
 	party_t	*pty;
 	long	n;
+	long	x, y;
 
 	dun = get_dun();
 	pty = get_party();
@@ -1175,25 +1308,16 @@ bool_t	appear_party_town( void )
 		clr_chr_trgt_act( pty->mbr[n], FALSE );
 	}
 
-	flg_find = FALSE;
-	for( i = 0; i < MAP_MAX_Y; i++ ){
-		for( j = 0; j < MAP_MAX_X; j++ ){
-			if( dun->map.obj.mjr[i][j]
-					== FACE_MJR_STAIRS_DOWN ){
-				flg_find = TRUE;
-				pty->square.x = j;
-				pty->square.y = i;
-				pty->square.pre_x = pty->square.x;
-				pty->square.pre_y = pty->square.y;
-				break;
-			}
-		}
-		if( flg_find ){
-			break;
-		}
-	}
-	if( !flg_find )
+	x = MAP_DEL_X;
+	y = MAP_DEL_Y;
+	if( !srch_map_obj( FACE_MJR_STAIRS_DOWN,
+			FACE_MNR_STAIRS_DOWN, &x, &y ) ){
 		return FALSE;
+	}
+	pty->square.x = x;
+	pty->square.y = y;
+	pty->square.pre_x = x;
+	pty->square.pre_y = y;
 
 	clr_map_all_mbr();
 	clr_map_all_pet();
@@ -1308,6 +1432,11 @@ bool_t	enter_shop( void )
 	clr_map_all_mbr();
 	clr_map_all_pet();
 
+	set_cur_shop_n( shop_n );
+	change_scene( SCENE_N_SHOP );
+
+	say_welcome_shop_msg( shop_n );
+
 	while( 1 ){
 		switch( face_mnr ){
 		case FACE_MNR_SHOP_INN:
@@ -1430,6 +1559,8 @@ void	party_exit_shop( long dr_n )
 		appear_all_pet();
 	}
 
+	next_pos_square( 0 );
+	scroll_center_appear_party();
 	set_flg_find_all_mnstr();
 	wipe_menu();
 
@@ -1437,6 +1568,202 @@ void	party_exit_shop( long dr_n )
 
 	if( dun->door[dr_n].mnr != FACE_MNR_SHOP_BAR )
 		add_turn( SHOP_ADD_TURN_N );
+
+	change_scene( SCENE_N_TOWN );
+}
+
+/**/
+
+void	say_welcome_shop_msg( shop_n_t shop_n )
+{
+	sex_t	sex;
+	const char	*name;
+
+	sex = get_npc_sex_shop( shop_n );
+	name = get_npc_name_shop( shop_n, sex );
+
+	switch( shop_n ){
+	case SHOP_N_NULL:
+	case SHOP_N_MAX_N:
+		break;
+	case SHOP_N_INN:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_INN_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_INN_MALE );
+		break;
+	case SHOP_N_BAR:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_BAR_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_BAR_MALE );
+		break;
+	case SHOP_N_WPN:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_WPN_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_WPN_MALE );
+		break;
+	case SHOP_N_ARMOR:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_ARMOR_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_ARMOR_MALE );
+		break;
+	case SHOP_N_MAGIC:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_MAGIC_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_MAGIC_MALE );
+		break;
+	case SHOP_N_TEMPLE:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_TEMPLE_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_TEMPLE_MALE );
+		break;
+	case SHOP_N_ALCHEMY:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_ALCHEMY_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_ALCHEMY_MALE );
+		break;
+	case SHOP_N_MUSIC:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_MUSIC_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_MUSIC_MALE );
+		break;
+	case SHOP_N_GROCERY:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_GROCERY_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_GROCERY_MALE );
+		break;
+	case SHOP_N_RESTAURANT:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_RESTAURANT_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_RESTAURANT_MALE );
+		break;
+	case SHOP_N_TEAROOM:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_TEAROOM_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_TEAROOM_MALE );
+		break;
+	case SHOP_N_TOBACCO:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_TOBACCO_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_TOBACCO_MALE );
+		break;
+	case SHOP_N_PET_SHOP:
+		print_words( name, sex,
+				MSG_WORDS_WELCOME_SHOP_PET_SHOP_FEMALE,
+				MSG_WORDS_WELCOME_SHOP_PET_SHOP_MALE );
+		break;
+	}
+}
+
+/**/
+
+const char	*get_npc_name_shop( shop_n_t shop_n, sex_t sex )
+{
+	if( sex == FEMALE ){
+		switch( shop_n ){
+		case SHOP_N_NULL:
+		case SHOP_N_MAX_N:
+			return MSG_NPC_NAME_SHOP_NULL_FEMALE;
+		case SHOP_N_INN:
+			return MSG_NPC_NAME_SHOP_INN_FEMALE;
+		case SHOP_N_BAR:
+			return MSG_NPC_NAME_SHOP_BAR_FEMALE;
+		case SHOP_N_WPN:
+			return MSG_NPC_NAME_SHOP_WPN_FEMALE;
+		case SHOP_N_ARMOR:
+			return MSG_NPC_NAME_SHOP_ARMOR_FEMALE;
+		case SHOP_N_MAGIC:
+			return MSG_NPC_NAME_SHOP_MAGIC_FEMALE;
+		case SHOP_N_TEMPLE:
+			return MSG_NPC_NAME_SHOP_TEMPLE_FEMALE;
+		case SHOP_N_ALCHEMY:
+			return MSG_NPC_NAME_SHOP_ALCHEMY_FEMALE;
+		case SHOP_N_MUSIC:
+			return MSG_NPC_NAME_SHOP_MUSIC_FEMALE;
+		case SHOP_N_GROCERY:
+			return MSG_NPC_NAME_SHOP_GROCERY_FEMALE;
+		case SHOP_N_RESTAURANT:
+			return MSG_NPC_NAME_SHOP_RESTAURANT_FEMALE;
+		case SHOP_N_TEAROOM:
+			return MSG_NPC_NAME_SHOP_TEAROOM_FEMALE;
+		case SHOP_N_TOBACCO:
+			return MSG_NPC_NAME_SHOP_TOBACCO_FEMALE;
+		case SHOP_N_PET_SHOP:
+			return MSG_NPC_NAME_SHOP_PET_SHOP_FEMALE;
+		}
+	} else {
+		switch( shop_n ){
+		case SHOP_N_NULL:
+		case SHOP_N_MAX_N:
+			return MSG_NPC_NAME_SHOP_NULL_MALE;
+		case SHOP_N_INN:
+			return MSG_NPC_NAME_SHOP_INN_MALE;
+		case SHOP_N_BAR:
+			return MSG_NPC_NAME_SHOP_BAR_MALE;
+		case SHOP_N_WPN:
+			return MSG_NPC_NAME_SHOP_WPN_MALE;
+		case SHOP_N_ARMOR:
+			return MSG_NPC_NAME_SHOP_ARMOR_MALE;
+		case SHOP_N_MAGIC:
+			return MSG_NPC_NAME_SHOP_MAGIC_MALE;
+		case SHOP_N_TEMPLE:
+			return MSG_NPC_NAME_SHOP_TEMPLE_MALE;
+		case SHOP_N_ALCHEMY:
+			return MSG_NPC_NAME_SHOP_ALCHEMY_MALE;
+		case SHOP_N_MUSIC:
+			return MSG_NPC_NAME_SHOP_MUSIC_MALE;
+		case SHOP_N_GROCERY:
+			return MSG_NPC_NAME_SHOP_GROCERY_MALE;
+		case SHOP_N_RESTAURANT:
+			return MSG_NPC_NAME_SHOP_RESTAURANT_MALE;
+		case SHOP_N_TEAROOM:
+			return MSG_NPC_NAME_SHOP_TEAROOM_MALE;
+		case SHOP_N_TOBACCO:
+			return MSG_NPC_NAME_SHOP_TOBACCO_MALE;
+		case SHOP_N_PET_SHOP:
+			return MSG_NPC_NAME_SHOP_PET_SHOP_MALE;
+		}
+	}
+
+	return MSG_NPC_NAME_SHOP_NULL_MALE;
+}
+
+/**/
+
+sex_t	get_npc_sex_shop( shop_n_t shop_n )
+{
+	switch( shop_n ){
+	case SHOP_N_NULL:
+	case SHOP_N_MAX_N:
+		break;
+	case SHOP_N_INN:
+		return MALE;
+	case SHOP_N_BAR:
+		return MALE;
+	case SHOP_N_WPN:
+		return MALE;
+	case SHOP_N_ARMOR:
+		return MALE;
+	case SHOP_N_MAGIC:
+		return MALE;
+	case SHOP_N_TEMPLE:
+		return MALE;
+	case SHOP_N_ALCHEMY:
+		return FEMALE;
+	case SHOP_N_MUSIC:
+		return FEMALE;
+	case SHOP_N_GROCERY:
+		return MALE;
+	case SHOP_N_RESTAURANT:
+		return MALE;
+	case SHOP_N_TEAROOM:
+		return FEMALE;
+	case SHOP_N_TOBACCO:
+		return FEMALE;
+	case SHOP_N_PET_SHOP:
+		return FEMALE;
+	}
+
+	return MALE;
 }
 
 /**/
@@ -1617,8 +1944,10 @@ void	stay_inn( bool_t flg_short_stay, bool_t flg_mews )
 
 		mbr = pty->mbr[i];
 
-		if( chk_flg_or( mbr->stat,
-				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+		if( chk_flg_or( mbr->stat, 0
+				| FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_STONE ) ){
 			continue;
 		}
 
@@ -1655,8 +1984,10 @@ void	stay_inn( bool_t flg_short_stay, bool_t flg_mews )
 	for( i = 0; i < PET_MAX_N; i++ ){
 		if( pet[i] == NULL )
 			continue;
-		if( chk_flg_or( pet[i]->stat,
-				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+		if( chk_flg_or( pet[i]->stat, 0
+				| FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_STONE ) ){
 			continue;
 		}
 
@@ -1695,7 +2026,13 @@ void	stay_inn( bool_t flg_short_stay, bool_t flg_mews )
 			get_calendar_str() );
 
 	if( chk_discount() ){
-		print_words( MSG_NPC_NAME_INN_LANDLORD, MALE,
+		sex_t	sex;
+		const char	*name;
+
+		sex = get_npc_sex_shop( SHOP_N_INN );
+		name = get_npc_name_shop( SHOP_N_INN, sex );
+
+		print_words( name, sex,
 				MSG_WORDS_DISCOUNT_DAY_FEMALE,
 				MSG_WORDS_DISCOUNT_DAY_MALE,
 				get_shop_name( g_discount_queue.shop_n ) );
@@ -1829,7 +2166,8 @@ bool_t	set_art_name_equip( mbr_t *mbr, item_t *art, equip_kind_t eq )
 		break;
 	}
 	if( eq_name != NULL ){
-		sprintf( art->art_name, MSG_FMT_ART_NAME_EQUIP,
+		sn_printf( art->art_name, ITEM_ART_NAME_MAX_LEN,
+				MSG_FMT_ART_NAME_EQUIP,
 				get_equip_ttl_max_len(), eq_name,
 				art->name );
 	} else {
