@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: trap.c,v 1.67 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: trap.c,v 1.71 2008/10/02 09:21:05 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -65,7 +65,8 @@
 #define	PIT_DAM_RATE	((rate_t)25)
 #define	PIT_DAM_ADD	1
 
-/* 櫤ǽͤΥ٥ */
+/* ƱᤵȾ¤ȽꤵΥ */
+#define	TRAP_NEAR_R	4
 
 /***************************************************************
 * Хѿ
@@ -190,6 +191,14 @@ void	init_trap( void )
 	trap_free.prev = &(trap_buf[i]);
 }
 
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_trap( void )
+{
+}
+
 /***************************************************************
 * ޥå׾櫤ƥꥻåȤ
 ***************************************************************/
@@ -213,16 +222,22 @@ void	reset_trap( void )
 
 trap_t	*make_trap( long x, long y, long dun_lev )
 {
-	long	i;
 	dun_t	*dun;
+	bool_t	flg_boss;
 	trap_t	*trap;
 	long	n;
+	long	i;
 
 	dun = get_dun();
 
-	if( clip_pos( x, y ) )
+	flg_boss = chk_boss_dun_lev( dun->lev );
+
+	if( clip_pos( x, y ) ){
 		if( dun->map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 			return NULL;
+		if( dun->map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+			return NULL;
+	}
 
 	n = 0;
 	for( i = LOOP_MAX_100; i > 0; i-- ){
@@ -242,6 +257,9 @@ trap_t	*make_trap( long x, long y, long dun_lev )
 
 		if( trap_tab[n].kind == TRAP_KIND_NULL )
 			continue;
+		if( trap_tab[n].kind == TRAP_KIND_CHUTE )
+			if( flg_boss )
+				continue;
 		if( !rate_randm( trap_tab[n].rate ) )
 			continue;
 		if( labs( dun_lev ) < trap_tab[n].min_lev )
@@ -373,6 +391,8 @@ bool_t	set_trap( long x, long y, trap_t *trap )
 		return FALSE;
 	if( dun->map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 		return FALSE;
+	if( dun->map.obj.mnr[y][x] != FACE_MNR_FLOOR )
+		return FALSE;
 
 	trap->x = x;
 	trap->y = y;
@@ -474,6 +494,8 @@ void	make_trap_room( long area_x, long area_y, long dun_lev )
 
 		if( dun->map.obj.mjr[my][mx] != FACE_MJR_FLOOR )
 			continue;
+		if( dun->map.obj.mnr[my][mx] != FACE_MNR_FLOOR )
+			continue;
 
 		trap = make_trap( mx, my, dun_lev );
 		if( trap != NULL )
@@ -1026,6 +1048,389 @@ trap_t	*get_trap( long x, long y )
 	return NULL;
 }
 
+/***************************************************************
+* 饯˰ֶᤤ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_nearest( chr_t *chr, act_kind_t act_kind )
+{
+	trap_t	*p, *ret_p;
+	long	min_r, tmp_r;
+	long	n;
+
+	if( chr == NULL )
+		return NULL;
+
+	min_r = MAP_MAX_X + MAP_MAX_Y;
+	ret_p = NULL;
+	n = 0;
+	for( p = dun_trap_asgn.next; p != &dun_trap_asgn; p = p->next ){
+		pos_t	pos1, pos2;
+
+		if( p == NULL )
+			break;
+		if( !chk_get_trap( chr, act_kind, p ) )
+			continue;
+
+		tmp_r = labs( p->x - chr->x ) + labs( p->y - chr->y );
+		if( tmp_r > min_r )
+			continue;
+
+		pos1.x = p->x;
+		pos1.y = p->y;
+		pos2.x = chr->x;
+		pos2.y = chr->y;
+		if( !chk_find( &pos1, &pos2 ) )
+			continue;
+
+		if( tmp_r < min_r ){
+			n = 0;
+			min_r = tmp_r;
+		} else if( tmp_r == min_r ){
+			n++;
+			if( randm( n ) != 0 )
+				continue;
+		}
+
+		ret_p = p;
+	}
+
+	return ret_p;
+}
+
+/***************************************************************
+* 饯˶ᤤ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_near( chr_t *chr, act_kind_t act_kind )
+{
+	trap_t	*p, *ret_p;
+	long	min_r, tmp_r;
+	long	n;
+
+	if( chr == NULL )
+		return NULL;
+
+	min_r = MAP_MAX_X + MAP_MAX_Y;
+	ret_p = NULL;
+	n = 0;
+	for( p = dun_trap_asgn.next; p != &dun_trap_asgn; p = p->next ){
+		pos_t	pos1, pos2;
+
+		if( p == NULL )
+			break;
+		if( !chk_get_trap( chr, act_kind, p ) )
+			continue;
+
+		tmp_r = labs( p->x - chr->x ) + labs( p->y - chr->y );
+		if( tmp_r > min_r )
+			continue;
+
+		pos1.x = p->x;
+		pos1.y = p->y;
+		pos2.x = chr->x;
+		pos2.y = chr->y;
+		if( !chk_find( &pos1, &pos2 ) )
+			continue;
+
+		if( tmp_r < min_r ){
+			n = 0;
+			min_r = tmp_r;
+		} else if( (tmp_r - min_r) <= TRAP_NEAR_R ){
+			n++;
+			if( randm( n ) != 0 )
+				continue;
+		}
+
+		ret_p = p;
+	}
+
+	return ret_p;
+}
+
+/***************************************************************
+* ɤΥ饯ޡƤʤ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_disperse( chr_t *chr, act_kind_t act_kind )
+{
+	trap_t	*p, *ret_p;
+	long	min_r, tmp_r;
+	long	n;
+	long	x, y;
+
+	if( chr == NULL )
+		return NULL;
+
+	x = chr->x;
+	y = chr->y;
+
+	min_r = MAP_MAX_X + MAP_MAX_Y;
+	ret_p = NULL;
+	n = 0;
+	for( p = dun_trap_asgn.next; p != &dun_trap_asgn; p = p->next ){
+		pos_t	pos1, pos2;
+
+		if( p == NULL )
+			break;
+		if( !chk_get_trap( chr, act_kind, p ) )
+			continue;
+
+		tmp_r = labs( p->x - x ) + labs( p->y - y );
+		if( tmp_r > min_r )
+			continue;
+
+		pos1.x = p->x;
+		pos1.y = p->y;
+		pos2.x = x;
+		pos2.y = y;
+		if( !chk_find( &pos1, &pos2 ) )
+			continue;
+
+		if( tmp_r < min_r ){
+			if( is_mbr( chr ) )
+				if( chk_already_mark( p, chr ) )
+					continue;
+			n = 0;
+			min_r = tmp_r;
+		} else if( tmp_r == min_r ){
+			if( is_mbr( chr ) )
+				if( chk_already_mark( p, chr ) )
+					continue;
+			n++;
+			if( randm( n ) != 0 )
+				continue;
+		}
+
+		ret_p = p;
+	}
+
+	return ret_p;
+}
+
+/***************************************************************
+* 饯˸Ƥ櫤˸
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_randm( chr_t *chr, act_kind_t act_kind )
+{
+	trap_t	*p, *ret_p;
+	long	n;
+
+	if( chr == NULL )
+		return NULL;
+
+	ret_p = NULL;
+	n = 0;
+	for( p = dun_trap_asgn.next; p != &dun_trap_asgn; p = p->next ){
+		pos_t	pos1, pos2;
+
+		if( p == NULL )
+			break;
+		if( !chk_get_trap( chr, act_kind, p ) )
+			continue;
+
+		pos1.x = p->x;
+		pos1.y = p->y;
+		pos2.x = chr->x;
+		pos2.y = chr->y;
+		if( !chk_find( &pos1, &pos2 ) )
+			continue;
+
+		/* Ĥ */
+		n++;
+		if( randm( n ) == 0 )
+			ret_p = p;
+	}
+
+	return ret_p;
+}
+
+/***************************************************************
+* ̤Ƥ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_identified( chr_t *chr, act_kind_t act_kind )
+{
+	return get_trap_identified_sub( chr, act_kind, TRUE );
+}
+
+/***************************************************************
+* ̤Ƥʤ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_un_identified( chr_t *chr, act_kind_t act_kind )
+{
+	return get_trap_identified_sub( chr, act_kind, FALSE );
+}
+
+/***************************************************************
+* ̾դ櫤򸡺
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* return : 
+***************************************************************/
+
+trap_t	*get_trap_identified_sub(
+	chr_t *chr, act_kind_t act_kind,
+	bool_t flg_identified
+)
+{
+	trap_t	*p, *ret_p, *sub_ret_p;
+	long	min_r, tmp_r;
+	long	n, sub_n;
+	long	x, y;
+
+	if( chr == NULL )
+		return NULL;
+
+	x = chr->x;
+	y = chr->y;
+
+	min_r = MAP_MAX_X + MAP_MAX_Y;
+	ret_p = NULL;
+	n = 0;
+	sub_ret_p = NULL;
+	sub_n = 0;
+	for( p = dun_trap_asgn.next; p != &dun_trap_asgn; p = p->next ){
+		pos_t	pos1, pos2;
+
+		if( p == NULL )
+			break;
+		if( !chk_get_trap( chr, act_kind, p ) )
+			continue;
+
+		if( flg_identified ){
+			if( !p->flg_chked )
+				continue;
+		} else {
+			if( p->flg_chked )
+				continue;
+		}
+
+		tmp_r = labs( p->x - x ) + labs( p->y - y );
+		if( tmp_r > min_r )
+			continue;
+
+		pos1.x = p->x;
+		pos1.y = p->y;
+		pos2.x = x;
+		pos2.y = y;
+		if( !chk_find( &pos1, &pos2 ) )
+			continue;
+
+		/* ƤǤ˥ޡѤλͽ */
+		sub_n++;
+		if( randm( sub_n ) == 0 )
+			sub_ret_p = p;
+
+		if( tmp_r < min_r ){
+			if( is_mbr( chr ) )
+				if( chk_already_mark( p, chr ) )
+					continue;
+			n = 0;
+			min_r = tmp_r;
+		} else if( tmp_r == min_r ){
+			if( is_mbr( chr ) )
+				if( chk_already_mark( p, chr ) )
+					continue;
+			n++;
+			if( randm( n ) != 0 )
+				continue;
+		}
+
+		ret_p = p;
+	}
+
+	if( ret_p == NULL )
+		ret_p = sub_ret_p;
+
+	return ret_p;
+}
+
+/***************************************************************
+* 櫤򸡺ǽĴ٤
+* chr_t *chr : 饯
+* act_kind_t act_kind : 
+* trap_t *p : 
+* return : ǽ?
+***************************************************************/
+
+bool_t	chk_get_trap( chr_t *chr, act_kind_t act_kind, trap_t *p )
+{
+	dun_t	*dun;
+
+	dun = get_dun();
+
+	if( p == NULL )
+		return FALSE;
+	if( !clip_pos( p->x, p->y ) )
+		return FALSE;
+	if( dun->map.obj.mjr[p->y][p->x] != FACE_MJR_TRAP )
+		return FALSE;
+
+	if( is_mbr( chr ) ){
+		if( !chk_flg( dun->map.obj.flg[p->y][p->x],
+				FLG_MAP_OBJ_FIND ) ){
+			return FALSE;
+		}
+		if( chk_flg( dun->map.obj.flg[p->y][p->x],
+				FLG_MAP_OBJ_LOOK_FLOOR ) ){
+			return FALSE;
+		}
+		if( chk_flg( dun->map.obj.flg[p->y][p->x],
+				FLG_MAP_OBJ_LOOK_WALL ) ){
+			return FALSE;
+		}
+		if( !chk_mark_trap( p, act_kind ) )
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+/***************************************************************
+* 櫤ޡ륢󤫥å
+* trap_t *p : 
+* act_kind_t act_kind : 
+* return : ޡ뤫?
+***************************************************************/
+
+bool_t	chk_mark_trap( trap_t *p, act_kind_t act_kind )
+{
+	if( p == NULL )
+		return FALSE;
+
+	switch( act_kind ){
+	case ACT_KIND_TRAP_DISARM:
+	case ACT_KIND_TRAP_BREAK:
+	case ACT_KIND_TRAP_CHK:
+		return TRUE;
+	default:
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
 /***************************************************************
 * 櫤Υǡơ֥򸡺
 * trap_kind_t kind : 櫤μ
